C Sedtrans05 - a sediment transport model
C Copyright (C) 2005 Urs Neumeier, Christian Ferrarin, Carl Amos
C and Georg Umgiesser.
C Sedtrans05 was developed at the ISMAR-CNR in Venice and the
C University of Southampton.
C Previous versions of Sedtrans (1992 and 1996) were developed at
C the Geological Survey of Canada, Bedford Institute of Oceanography.
C
C This program is free software; you can redistribute it and/or modify
C it under the terms of the GNU General Public License as published by
C the Free Software Foundation; either version 3 of the License, or
C (at your option) any later version. See file "licence.txt" in the
C Sedtrans05 directory for the full licence text, which can also be
C found at http://www.gnu.org/licenses/
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C **********************************************************************
      PROGRAM SEDI1DT
      IMPLICIT NONE
      
      CHARACTER*12 VERSION /'version 1.04'/
      CHARACTER*80 INFILE,OUTFILE,CONSTFILE,LOGFILE
      INTEGER NBSTEP          ! number of time steps
      INTEGER TIME            ! time in seconds from the start of the simulation
      DOUBLE PRECISION ZBED   ! bed elevation refered to initial position  ! vers. 1.04 renamed Z to ZBED
                              ! positive upward                            ! to avoid confusion with Z of UZ
      INTEGER MODE           ! see comment below (before opening file 10
      INTEGER DT              ! duration in seconds of the hydrodynamic condtions
      DOUBLE PRECISION TAUCWS ! bed shear stress read from input file (Pa)
      DOUBLE PRECISION SSCMES ! measured SSC, read from input file (Pa)
      DOUBLE PRECISION OBS2   ! SSC outside of the flume measured by OBS2 (kg/m3)
      DOUBLE PRECISION ROT    ! rotation speed of lid (m/s)
      DOUBLE PRECISION TEMP   ! temperature for viscosity calculation
      DOUBLE PRECISION SALIN  ! salinity for viscosity calculation
      INTEGER ERR(20)/20*0/   ! error code
      INTEGER I

      INCLUDE "sed_param.h" ! declaration of the COMMON block /WSCLASS/

C INPUT VARIABLES to COHESIVE
      DOUBLE PRECISION CONC(NBCONC) !array of suspended sediment concentration (mg/l)
                              ! for each Ws class,
                              ! Ws of CONC(i) = 10**((i-26)/5)          user input
      INTEGER MAXBED          ! upper bound of first dimension of BEDCHA
      PARAMETER (MAXBED=20)
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
                              ! (1) depth below sediment surface (m)
                              ! (2) critical erosion stress (Pa)
                              ! (3) dry bulk density (kg/m**3)
      DOUBLE PRECISION TIMEDR !Duration of call to Sedtrans (s)
      DOUBLE PRECISION USTCWS !Combined skin friction stress
      DOUBLE PRECISION U100   !current speed at 1m above seabed (m/sec)
      DOUBLE PRECISION D      !water depth (m)
      DOUBLE PRECISION RHOW   !density of fluid (water)  (kg/m**3)
      DOUBLE PRECISION VISC   !(dynamic or molecular) viscosity (kg/(s*m))
      DOUBLE PRECISION AULVA  !percentage of area covered by Ulva (%)

C OUTPUT VARIABLES from COHESIVE
      DOUBLE PRECISION EDRATE  !mean erosion/deposition rate (kg/m^2/s)
                               !(positive: erosion, negative: deposition
      DOUBLE PRECISION ZS      !Height change in bed surface (m)
                               !(positive: erosion, negative: deposition)
      DOUBLE PRECISION SEDM    !time-averaged net sediment transport as mass (kg/s/m)
      DOUBLE PRECISION TCONC   !final total suspended sediment concentration (kg/m**3)
      DOUBLE PRECISION TAOS    !solid transmitted stress due to Ulva (Pa)

C Other input variables to Sedtrans05
      DOUBLE PRECISION UZ,Z,CDIR,HT,PER,
     &         WDIR,GD,RHINP,RLINP,BETA,RHOS
      INTEGER IOPT1
C Other output variables to Sedtrans05
      DOUBLE PRECISION
     &UB,FCW,UA,USTCW,Z0,RHEIGHT,RLENGTH,QS,QSDIR,
     &SED,SEDDIR,FALL,C0A,
     @USTCRB,USTCRS,USTUP



      CALL INICONST ! initialize the constants in the common block CCONST

      INFILE='in1d.dat' ! default name for the input file
      OUTFILE='out1d'   ! default name for the output file
      CONSTFILE=' '     ! file to modify the default parameters
C If the functions retrieving the command-line arguments are not suported
C by the compiler, comment out the line CALL COMARG...
      CALL COMARG2(INFILE,OUTFILE,CONSTFILE,VERSION)  ! read input/output file-name on the command line

      CALL OPENFILES(INFILE,OUTFILE,
     & MODE,D,SALIN,TEMP,CONC,MAXBED,BEDCHA)        ! open the files

      IF(CONSTFILE.NE.' ')
     &    CALL SETCCONST(CONSTFILE,15,' ')   ! modify default constant if requested

C      open (20,file='output.txt')    !  debugging ouput

      IF(MODE.GE.10)THEN
C set invariable parameters
         U100=1d0
         AULVA=0
         SSCMES=0  ! For MODE=10
         CALL DENSVISC(TEMP,SALIN,RHOW,VISC) ! compute the viscosity/density
      ENDIF

C initialize some variables
      TIME=0
      ZBED=0d0                                                          ! vers. 0.80
      TCONC=0d0
      DO 30,I=1,NBCONC
         TCONC=TCONC+CONC(I)
30    CONTINUE
      NBSTEP=-1

50    CONTINUE     ! start of MAIN LOOP
C read one input line and check values
         IF(MODE.GE.10) THEN
            CALL INPUT1(MODE,ERR,DT,TAUCWS,SSCMES,OBS2,ROT)
            IF(ERR(1).EQ.-1)GOTO 90
         ELSE
60          READ(10,*,ERR=60,END=90)DT,IOPT1,D,UZ,Z,CDIR,HT,PER,
     &         WDIR,GD,RHINP,RLINP,BETA,RHOS,SALIN,TEMP,TIMEDR,AULVA
            IOPT1=7
            GD=50d-6
            RHOS=2650d0
            SSCMES=0d0
            CALL CHECKIN(ERR,D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,
     &         RLINP,BETA,RHOS,SALIN,TEMP,7,3d2,AULVA,
     &         BEDCHA,MAXBED,CONC)
            IF(ERR(1).EQ.0)
     &         CALL DENSVISC(TEMP,SALIN,RHOW,VISC) ! compute the viscosity/density
         ENDIF


         IF (NBSTEP.EQ.-1) THEN
C write initial values to output file
            CALL OUTPUT1(0,ERR,0d0,ZBED,0d0,0d0,TCONC,                  ! vers. 0.80
     &                     CONC,MAXBED,BEDCHA,SSCMES)
         ENDIF

         IF(ERR(1).NE.0) THEN ! compute the number of sub-loops
            NBSTEP=1
            TIMEDR=0
         ELSEIF(DT.LE.60 .OR. MODE.EQ.11) THEN
            NBSTEP=1
            TIMEDR=DBLE(DT)
         ELSE
            NBSTEP=DT/60
            TIMEDR=60d0
         ENDIF

         DO 80, I=1,NBSTEP  ! start of LOOP 2
            TIME=TIME+INT(TIMEDR)

            IF(ERR(1).NE.0)THEN
               CONTINUE    ! do nothing if error in input values
            ELSEIF(MODE.GE.10)THEN

               USTCWS=(TAUCWS/RHOW)**5d-1
               CALL COHESIVE (CONC,BEDCHA,MAXBED,TIMEDR,USTCWS,U100,D,
     &                     RHOW,VISC/RHOW,AULVA,
     &                     EDRATE,ZS,SEDM,TCONC,TAOS)
C correction of diffusion out of the Sea Carousel
               IF(MODE.EQ.14)
     &            CALL SCARDIFF(TCONC,CONC,TIMEDR,SSCMES,OBS2,ROT)

            ELSE  ! MODE < 10
            
               RHEIGHT=RHINP
               RLENGTH=RLINP
               CALL SEDTRANS05(            ! call to the core-part of Sedtrans
     @D,UZ,Z,CDIR,HT,PER,WDIR,GD,
     @BETA,RHOS,RHOW,IOPT1,VISC,CONC,BEDCHA,MAXBED,TIMEDR,AULVA, !**INPUT VARIABLES**                                					   	!**INPUT VARIABLES**
     @
     @USTCRB,USTCRS,USTUP,UB,FCW,UA,U100,USTCWS,USTCW,Z0,RHEIGHT,
     @RLENGTH,QS,QSDIR,SEDM,SED,SEDDIR,FALL,C0A,EDRATE,ZS,TAOS,
     @TCONC)		!**OUTPUT VARIABLES**

               TAUCWS=(USTCWS**2d0)*RHOW                                ! vers. 0.80
            ENDIF
     
            IF(ERR(1).EQ.0)THEN                                         ! vers. 0.80
               ZBED=ZBED-ZS
            ENDIF

C write result in output files
            CALL OUTPUT1(TIME,ERR,TAUCWS,ZBED,ZS,EDRATE,TCONC,
     &                    CONC,MAXBED,BEDCHA,SSCMES)
            IF(MODE.LT.10) THEN
               CALL OUTPUT2(TIME,ERR,USTCWS,UB,FCW,UA,U100,
     &            USTCW,Z0,RHEIGHT,RLENGTH,QS,QSDIR,SEDM,
     &            SED,SEDDIR,FALL,C0A,RHOW,VISC,USTCRB,USTCRS,USTUP)
            ENDIF

80       CONTINUE   ! end of loop 2
      GOTO 50       ! end of MAIN LOOP

90    CLOSE(10)
      CLOSE(11)
      CLOSE(12)
      CLOSE(13)
      CLOSE(14)
      CLOSE(15)

      END    ! end of main program SEDI1DT

      

C ======================================================================
C ======================================================================
C SUBROUTINE COMARG2
C
C Read the arguments on the command line.
C Print instructions on the screen if command-line argument is -? , then stop
C Generate an example input-file "INDATA.DAT" if command line argument
C is -e, then stop.
C Print short error message when command-line argument is not recognized, then stop.

      SUBROUTINE COMARG2(INFILE,OUTFILE,CONSTFILE,VERSION)

      IMPLICIT NONE
      CHARACTER INFILE*(*) ! Name of input file (batch mode)
      CHARACTER OUTFILE*(*)! Name of output file
      CHARACTER CONSTFILE*(*)! Name of file to modify the default parameters
      CHARACTER VERSION*(*)! version of the program

      CHARACTER PROGNAME*6/'SEDI1D'/
      INTEGER NBARG        ! Number of command-line argument
      CHARACTER ARG*200    ! command-line argument
      INTEGER I,POS,LASTCH

C This routine uses two non stantard functions to retrieve the argument
C from the command line:
C nb_argument = IARGC()
C CALL GETARG(n,argument_number_n)

      NBARG=IARGC()
      IF (NBARG.EQ.0) THEN
         RETURN
      ENDIF
      CALL GETARG(1,ARG)
      IF (ARG.EQ.'-?' .OR. ARG.EQ.'/?' .OR. ARG.EQ.'-c' .OR.
     &      ARG.EQ.'-C') THEN
         WRITE(*,100)VERSION,(PROGNAME,I=1,7)
         STOP
      ELSEIF (ARG.EQ.'-l' .OR. ARG.EQ.'-L') THEN
         WRITE(*,701)VERSION
         WRITE(*,702)
         STOP
      ELSEIF (ARG.EQ.'-E' .OR. ARG.EQ.'-e')THEN
         CALL EXAMPLE2          ! generate example files
         CALL SETCCONST(' ',0,' ')  ! generate example file to modify default parameters
         STOP
      ENDIF

      CONSTFILE=' '
      INFILE=ARG
      IF(NBARG.GT.1)THEN
         CALL GETARG(2,OUTFILE)
         IF(NBARG.GT.2) THEN
            CALL GETARG(3,CONSTFILE)
         ENDIF
      ENDIF

C for batch mode, use same input filename for output if no name conflicts
      IF(NBARG.EQ.1)THEN
         POS=LASTCH(INFILE)
         ARG=INFILE(MAX(1,POS-3):POS)
         DO 20,I=1,4
            IF(ARG(I:I).GE.'A'.AND.ARG(I:I).LE.'Z')
     &         ARG(I:I)=CHAR(ICHAR(ARG(I:I))+32)
20       CONTINUE
         IF(ARG.NE.'.dat'.AND.ARG.NE.'.tab'.AND.ARG.NE.'.log') THEN
            POS=INDEX(INFILE,'.')
            IF(POS.GT.1)THEN
               OUTFILE=INFILE(1:POS-1)
               GOTO 40
            ENDIF
         ENDIF
      ENDIF
40    CONTINUE

100   FORMAT(/,
     &'Command-line syntax for using the 1D-time version of Sedtrans05',
     &' (',A,')',//,
     &A,' -?  Show the present help text'/,
     &A,'     By default the input file is "IN1D.DAT"'/,
     &'           and output files are "OUT1D.*", ',//
     &'The files can also be specified as follow:'/,
     &A,' inputfile'/,
     &A,' inputfile outputfile'/,
     &A,' inputfile outputfile paramfile     Modify the default',
     &' parameters ',/,
     &T46,'according to the file "paramfile"',//,
     &'"inputfile" must be with extension, "outputfile must be without',
     &' extension.',//,
     &A,' -L',T12,'Show the licence',/,
     &A,' -e',T12,'Generate the example files "IN1D.DAT" (input data)'
     &,' and ',/,
     &T12,'"INDATA.CST" (modification of default parameters), and ',
     &'write the ',/,
     &T12,'list of abbreviations used in the output files to the ',
     &'file',/,
     &T12,'"OUT1D.LST"',/)

C The same formats 701 and 702 exist in COMARG and in OPENFILE, they
C must be updated together !
701   FORMAT(
     &/,'1D time model using the Cohesive Sediment algorithm of ',
     &'SEDTRANS05 (',A,', test version)',/
     &/,'Copyright (C) 2005 Urs Neumeier, Christian Ferrarin, Carl Amos'
     &/,'and Georg Umgiesser.',
     &/,'Sedtrans05 was developed at the ISMAR-CNR in Venice and the',
     &/,'University of Southampton.',
     &/,'Previous versions of Sedtrans (1992 and 1996) were developed',
     &' at',
     &/,'the Geological Survey of Canada, Bedford Institute of ',
     &'Oceanography.',/)
702   FORMAT(
     &'This program is free software; you can redistribute it and/or',
     &' modify',
     &/,'it under the terms of the GNU General Public License as ',
     &'published by',
     &/,'the Free Software Foundation; either version 3 of the ',
     &'License, or',
     &/,'(at your option) any later version. See file "licence.txt" ',
     &'in the'
     &/,'Sedtrans05 directory for the full licence text, which can ',
     &'also be',
     &/,'found at http://www.gnu.org/licenses/',
     &/,
     &/,'This program is distributed in the hope that it will be ',
     &'useful,',
     &/,'but WITHOUT ANY WARRANTY; without even the implied warranty of'
     &/,'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the',
     &/,'GNU General Public License for more details.')


      END      ! end of subroutine COMARG



C ======================================================================
C ======================================================================
C SUBROUTINE EXAMPLE2
C
C Generate an input example-file (IND1.DAT) and a file (OUT1D.LST)
C with the list of abbreviations used in the output files

      SUBROUTINE EXAMPLE2
      IMPLICIT NONE
      INCLUDE "sed_param.h" ! declaration of the COMMON block /WSCLASS/
      CHARACTER NAME*20,CH*500
      LOGICAL EX
      INTEGER I,LASTCH
      CHARACTER*140 ERRTXT ! Error/Warning text corresponding to ERR, if the
                           ! text is longer than 75 characters, ERRTXT(1:75) is
                           ! the 1st line and ERRTXT(76:140) is the second line
      NAME='IN1D.DAT'
      INQUIRE(FILE=NAME, EXIST=EX)
      IF (EX) THEN
         WRITE(*,102) NAME(1:LASTCH(NAME))
         READ(*,*) CH
         IF (CH.NE.'Y' .AND. CH.NE.'y') THEN
            WRITE(*,103) NAME(1:LASTCH(NAME))
            GOTO 10
         ENDIF
      ENDIF
      OPEN(10,FILE=NAME,ERR=20)

      WRITE(10,120,ERR=20)
      WRITE(CH,121)(I,I=1,21),(I,I=1,4),(I,I=1,4),(I,I=1,4)
      CALL LINETAB(CH)
      WRITE(10,'(A)',ERR=20)CH(1:LASTCH(CH))
      WRITE(CH,122)
      CALL LINETAB(CH)
      WRITE(10,'(A)',ERR=20)CH(1:LASTCH(CH))
      WRITE(10,123,ERR=20)
      WRITE(10,124,ERR=20)
      WRITE(CH,125)
      CALL LINETAB(CH)
      WRITE(10,'(A)',ERR=20)CH(1:LASTCH(CH))
      WRITE(CH,126)
      CALL LINETAB(CH)
      WRITE(10,'(A)',ERR=20)CH(1:LASTCH(CH))
      WRITE(10,127,ERR=20)
      CLOSE(10,ERR=20)

10    NAME='OUT1D.LST'
      INQUIRE(FILE=NAME, EXIST=EX)
      IF (EX) THEN
         WRITE(*,102) NAME(1:LASTCH(NAME))
         READ(*,*) CH
         IF (CH.NE.'Y' .AND. CH.NE.'y') THEN
            WRITE(*,106) NAME(1:LASTCH(NAME))
            RETURN
         ENDIF
      ENDIF
      OPEN(10,FILE=NAME,ERR=40)

      WRITE(10,130,ERR=40)
      WRITE(10,131,ERR=40)
      WRITE(10,132,ERR=40)
      CALL INICONST
      WRITE(10,133,ERR=40)(I,WSI(I),I=1,NBCONC)
      WRITE(10,134,ERR=40)
      DO 15,I=1,107
         IF(I.EQ.100)WRITE(10,135,ERR=40)
         CALL ERRTEXT(I,ERRTXT)
         IF(ERRTXT.NE.' ')THEN
            IF (LASTCH(ERRTXT).LE.75) THEN
               WRITE(10,136,ERR=40)I,ERRTXT(1:LASTCH(ERRTXT))
            ELSE
               WRITE(10,137,ERR=40)I,ERRTXT(1:75),
     &               ERRTXT(76:LASTCH(ERRTXT))
            ENDIF
         ENDIF
15    CONTINUE

      CLOSE(10)
      RETURN

20    WRITE(*,104) NAME(1:LASTCH(NAME))
      CLOSE (10,ERR=30)
30    GOTO 10
40    WRITE(*,104) NAME(1:LASTCH(NAME))
      CLOSE (10,ERR=50)
50    RETURN

102   FORMAT (/'The existing file ',A,' will be overwritten, '/,
     &'do you want to continue ? (Y/N)')
103   FORMAT (/'Generation of example ',A,' aborted.')
104   FORMAT (/'Error during opening/writing file ',A,/)
106   FORMAT (/'Generation of file ',A,' aborted.')

120   FORMAT('Example input file for SEDI1D, the 1D-time model for the'/
     &'cohesive sediment algorithm of Sedtrans05.',//,
     &'The input file can have different formats, depending the first ',
     &/,'parameter on the first data line (parameter MODE).',//,
     &'The first line of the file is not processed, but written at the'/
     &'top of the .dat output file, it should be a comment. Other ',/,
     &'comment lines (with any text) are also ignored.',//,
     &'The first data line must be')
121   FORMAT('MODE WATER_DEPTH SALINITY TEMPERATURE',
     &9(' c',I1),12(' c',I2),
     &' NBED',4(' z',I1),4(' Tero',I1),4(' rho',I1))
122   FORMAT('14 0.5 35. 20.',21(' 0.0'),' 4 ',
     &'0.0 0.005 0.02 0.05 ',
     &'0.2 0.3 0.4 0.45 ',
     &'200. 300. 400. 450.')
123   FORMAT(/,'If MODE = 10-15, then input file has FORMAT A, and ',/,
     &'subroutine COHESIVE is called directly without passing through',/
     &'subroutine SEDTRANS05 ',/,
     &'If MODE < 10, then input file has FORMAT B, and the program call'
     &/'subroutine SEDTRANS05, which itself call subroutine COHESIVE.'//
     &'Each of the successive data lines specifies the hydrodynamic ',/
     &'conditions prevailing for defined duration DT. DT (long integer)'
     &,/,'is in seconds. DT of each line may be different.',/)
124   FORMAT('Format MODE=10',/,
     &'DT, TAUCWS',/
     &'      all values after TAUCWS are ignored',//
     &'Format MODE=11',/,
     &'      As MODE=10, but the time step of the model and of the ',/,
     &'      output files is always the values DT (and not 60 seconds).'
     &,/,
     &'      This is for the calibration of the consolidation ',/,
     &'      algorithm.',//,
     &'Format MODE=12',/
     &'DT, TAUCWS, SSCmeasured',
     &'      SSCmeasured is added at the end of the output line',//,
     &'Format MODE=14',
     &'DT, TAUCWS,SSCmeasured,OBS2,ROT',/
     &'      as MODE=12,',
     &'      and suspended sediment diffusing out of the Sea Carousel',/
     &'      is computed',/,
     &'      both SSCmeasured and SSC predicted are modified'//
     &'Format MODE=7',/,
     &'In this format, values 2-4 of the first data line (water depth,'/
     &'salinity, temperature) are ignored.',/,
     &'The following lines are similar than for the sed05 batch mode.',/
     &'However, the first column is the durations (seconds), and ',/,
     &'arguments iopt1, timedr and gd are ignored.'//)
125   FORMAT('DT iopt1 D UZ Z CDIR HT  PER WDIR gd RHINP RLINP BETA ',
     &'RHOS SALIN TEMP timedr AULVA')
126   FORMAT('120 0 2.2 0.8 0.5 30.5 0.3 3.5 185. 0 0.0 0.0 0.0 ',
     &'0. 35. 15. 0 0.')
127   FORMAT(/,'Meaning of the abbreviations',/,
     &'DT',T10,'duration of specific hydrodynamic conditions ',
     &'(s., integer value, if DT>60, must by multiple of 60)',/,
     &'IOPT1',T10,'not used',/,
     &'D',T10,'water depth (m)',/,
     &'UZ',T10,'current speed (m/s)',/,
     &'Z',T10,'height of current measurement (m)',/,
     &'CDIR',T10,'current direction (degrees north)',/,
     &'HT',T10,'wave height (m)',/,
     &'PER',T10,'wave period (seconds)',/,
     &'WDIR',T10,'wave direction (degrees north)',/,
     &'GD',T10,'not used',/,
     &'RHINP',T10,'ripple height (m)',/,
     &'RLINP',T10,'ripple length (m)',/,
     &'BETA',T10,'bed slope (degree)',/,
     &'RHOS',T10,'not used',/,
     &'SALIN',T10,'salinity (default=35)',/,
     &'TEMP',T10,'temperature (degrees Celsius)',/,
     &'TIMEDR',T10,'not used',/,
     &'AULVA',T10,'fraction of bed area covered by Ulva (range 0-1)',/,
     &'c1-c21',T10,'suspended sediment concentration of each Ws class'
     &,/,'NBED',T10,'number (N) of entries in BEDCHA',/,
     &'z1-zN',T10,'depth (positive downward, m)',/,
     &'rho1-rhoN  dry bulk density (kg/m**3)',/,
     &'Tero1-TeroN  critical erosion stress (Pa)')
C List of output abbreviations for the file OUTPUT.LST
130   FORMAT('Abbreviations used in the output files of Sedi1d ',
     &   '(1D-time version of Sedtrans05 for cohesive sediments)',/,/,
     &'time',T9,'Time from start (s)',/,
     &'Error',T9,'error/warning code (see below)',/,
     &'Tau0eff',T9,'effective To, includes drag reduction and ',
     &   'solid transmitted stress due to Ulva Pa)',/,
     &'TauCWS',T9,'combined (wave-current) skin-friction shear stress',
     &   ' (Pa)',/,
     &'z',T9,'bed elevation refered to initial position, ',
     &   'positive upward (m)',/,
     &'dz',T9,'height change in bed surface (m)',/,
     &T9,'(positive: erosion, negative: deposition)',/,
     &'edrate',T9,'mean erosion/deposition rate (kg/m**2/s)',/,
     &T9,'(positive: erosion, negative: deposition)',/,
     &'tconc',T9,'total suspended sediment concentration (kg/m**3)',/,
     &'nbconc',T9,'number of classes of suspended sediment',/,
     &'cN',T9,'suspended sediment concentration for each class (kg/m3)'
     &    ,/,
     &'bed characteristics (BEDCHA):'/,
     &'maxbed',T9,'number of elements in z, Te and r',/,
     &'zN',T9,'depth below surface (m)',/,
     &'TeN',T9,'critical erosion stress at depth zN (Pa)',/,
     &'rN',T9,'dry bulk density at depth zN (kg/m**3)',/,
     &/,
     &'In the output file *.tab',/,/,
     &'Same content than in the *.dat file, but without header line',//)
131   FORMAT('In the output file *.da1 (similar to *.dat file of sed05)'
     &   ,', only for MODE=7',/,/,
     &'DT',T9,'Time from start (s)',/,
     &'ERR',T9,'error/warning code (see below)',/,
     &'UB',T9,'maximum wave induced orbital velocity at the bottom ',
     &'(m/s)',/,
     &'FCW',T9,'bottom (skin) friction factor',/,
     &'UA',T9,'current speed to be used in bottom stress calc. (m/s)'/
     &'U100',T9,'current speed at 1 m. above seabed (m/s)',/,
     &'USTCWS',T9,'combined skin-friction shear velocity of GM (m/s)'/
     &'USTCW',T9,'combined total shear velocity of GM (m/s)',/,
     &'Z0',T9,'bed roughness length (m)',/,
     &'FALL',T9,'settling velocity for non-cohesive sediment (m/s)',/,
     &'RHEIGHT',T9,'ripple height, same as input (m)',/,
     &'RLENGTH',T9,'ripple length, same as input (m)',/,
     &'C0A',T9,'depth averaged reference concentration at z0 (kg/m^3)'/
     &'QS',T9,'suspended sediment transport rate (kg/m/s)',/,
     &'QSDIR',T9,'direction of suspended sediment transport (degree)',/,
     &'SED',T9,'time-averaged net sediment transport as volume ',
     &'(m**3/s/m)',/,
     &'SEDM',T9,'time-averaged net sediment transport as mass (kg/s/m)'/
     &'SEDDIR',T9,'direction of net sediment transport ',
     &'(azimuth,degrees)',//)
132   FORMAT('In the output file *.da2 (additional parameters)',
     &   ', only for MODE=7',/,/,
     &'IRUN',T9,'Time from start (s)',/,
     &'AB',T9,'excursion length of bottom wave orbit (m)',/,
     &'WL',T9,'wave length (m)',/,
     &'USTCRB',T9,'critical shear vel for initi of bedload trans ',
     &'(m/s)',/,
     &'USTCRS',T9,'critical shear vel for initi of suspended load ',
     &'transport (m/s)',/,
     &'USTUP',T9,'critical shear vel for initn of sheet flow ',
     &'transport (m/s)',/,
     &'Z0C',T9,'apparent bed roughness length (m)',/,
     &'PHIB',T9,'angle between wave and current directions (radians)',/,
     &'PHI100',T9,'angle between wave and current directions at ',
     &'1 m. above seabed',/,
     &'DELTACW',T9,'height of the wave-current boundary layer',/,
     &'USTCS',T9,'current skin-friction shear velocity of GM',/,
     &'USTWS',T9,'wave skin-friction shear velocity of GM',/,
     &'USTCWSE',T9,'effective combined skin-friction shear velocity',/,
     &'USTCWSB',T9,'transport-related combined shear velocity',/,
     &'USTC',T9,'total current shear velocity of GM',/,
     &'USTW',T9,'total wave shear velocity of GM',/,
     &'RPLCOEF',T9,'ripple coefficient for shear velocity conversion',/,
     &'TB1',T9,'time at which bedload transport ceases (s)',/,
     &'TB2',T9,'time at which bedload transport recommences (s)',/,
     &'TS1',T9,'time at which suspended load transport ceases (s)',/,
     &'TS2',T9,'time at which suspended load transport ',
     &'recommences (s)',/,
     &'PERBED',T9,'percentage of time spent in only bedload ',
     &'transport phase',/,
     &'PERSUSP',T9,'percentage of time spent in suspended load ',
     &'transport phase',/,
     &'C0',T9,'reference concentration at z0 (kg/m^3)',/,
     &'TAOCWS',T9,'averaged effective shear stress',/,
     &'RHOW',T9,'water density (kg/m3)',/,
     &'VISC',T9,'dynamic viscosity of water (Pa s)',//)
133   FORMAT(//,'Settling velocities (Ws) for each suspended sediment',/
     &,'class of cohesive sediment',/,
     &'Class  Ws (m/s)',/,21(I4,1X,F10.6,/))
134   FORMAT(/,'Error and warning codes',/
     &'Only one error/warning code is specified in the file *.dat',/,
     &'The full list of error/warning is given in the file *.txt',//,
     &'  0 No error or warning',//,
     &'Critical errors',/)
135   FORMAT(/,'Warnings',/)
136   FORMAT(I3,1X,A)
137   FORMAT(I3,1X,A,/,4X,A)

      END   ! end of subroutine EXAMPLE2



C ======================================================================
C ======================================================================
C SUBROUTINE OPENFILES
C
C Open the input file and the output files
C Read the initial conditions and return them as output argument
C See detailed description of file-format below
C
      SUBROUTINE OPENFILES(INFILE,OUTFILE,
     & MODE,D,SALIN,TEMP,CONC,MAXBED,BEDCHA)

      IMPLICIT NONE
      INCLUDE 'sed_param.h'

      CHARACTER*80 INFILE,OUTFILE1,OUTFILE2,OUTFILE3,OUTFILE4,OUTFILE5
      CHARACTER*80 LINE1
      CHARACTER OUTFILE*(*)
      CHARACTER*2000 CH       ! used for formatting output
      INTEGER MODE,I,J,MAXBED
      INTEGER NBED            ! number of row in BEDCHA read from input file
      DOUBLE PRECISION D,SALIN,TEMP, TMP
      DOUBLE PRECISION CONC(NBCONC)
      DOUBLE PRECISION BEDCHA(MAXBED,3)
      LOGICAL CHECKBEDCHA ! function
      INTEGER LASTCH      ! function

C Read initial conditions in the input file
C The input file can have different formats, depending the first parameter
C on the first data line (parameter MODE)
C
C The first line of the file is not processed, but written at the top of the
C .dat output file, it should be an comment. Other comment lines
C (with any text) are also ignored.
C
C The first data line must be
C MODE,water depth, salinity, temperature, CONC (21 elements), NBED, ((BEDCHA(I,J),I=1,NBED),J=1,3)
C
C If MODE = 10-15, then input file has FORMAT A, and subroutine COHESIVE is
C called directly without passing through subroutine SEDTRANS05
C If MODE < 10, then input file has FORMAT B, and the program call subroutine
C SEDTRANS05, which itself call subroutine COHESIVE
C
C Each of the succesive data lines specifies the hydrodynamic conditions
C prevaling for defined duration DT. DT (long integer) is in seconds.
C DT of each line may be different.
C
C Format MODE=10
C DT, TAUCWS
C          all values after TAUCWS are ignored
C
C Format MODE=11
C          As MODE=10, but the time step of the model and of the output
C          files is always the values DT (and not 60 seconds).
C          This is for the calibration of the consolidation algorithm.
C
C Format MODE=12
C DT, TAUCWS, SSCmeasured
C          SSCmeasured is added at the end of the ouput line
C
C Format MODE=14
C DT, TAUCWS,SSCmeasured,OBS2,ROT
C          as MODE=12,
C          and suspended sediment diffusing out of the SeaCarousel is computed
C          both SSCmeasured and SSC predicted are modified
C
C
C Format MODE=7
C In this format, values 2-4 of the first data line (water depth, salinity,
C temperature) are ignored.
C The following lines are similar than for the sed05 batch mode.However,
C the first column is the durations (seconds), and following arguments are
C ignored: iopt1, timedr, gd
C
C DT,iopt1,D,UZ,Z,CDIR,HT,PER,WDIR,gd,RHINP,RLINP,BETA,RHOS,SALIN,TEMP,timedr,AULVA
C
C
C
C ========
C If NBED=1, then BEDINI is called with either BEDCHA(1,2) or BEDCHA(1,3)
C x1 (real value) is simply written in the ouput if present.
C
C If a duration is longer than 60s, then it must be a multiple of 60s.
C
C The output will be at time-steps of 60s, or of the duration if shorter than 60s
C
      OPEN (10,FILE=INFILE,STATUS='OLD',ERR=91)
      READ(10,'(80A)',END=90)LINE1    ! read first line that must be a comment line
10    READ(10,*,ERR=10,END=90)MODE,D,SALIN,TEMP,CONC,NBED,
     &                  ((BEDCHA(I,J),I=1,MIN(NBED,MAXBED)),J=1,3)

      IF (NBED.GT.MAXBED) THEN
         WRITE(*,*) 'NBED must not be larger than ',MAXBED,'.'
         STOP
      ELSEIF ((D.LE.0).AND.(MODE.GE.10)) THEN
         WRITE(*,*) 'D must be greather than 0.'
         STOP
      ENDIF
      TMP=0d0
      DO 15,I=15,NBCONC
         IF(CONC(I).LT.0)THEN
            WRITE(*,*) 'All elements of CONC must be greather than 0.'
            STOP
         ENDIF
         TMP=TMP+CONC(I)
15    CONTINUE
      IF(TMP.GT.100d0)THEN
         WRITE(*,*) 'The total of CONC is too large.'
         STOP
      ENDIF

      IF(NBED.EQ.1) THEN
         IF(BEDCHA(1,2).GT.0) THEN
            TMP=BEDCHA(1,2)
         ELSEIF(BEDCHA(1,3).GT.0) THEN
            TMP=BEDCHA(1,3)
         ELSE
            WRITE(*,*)'BEDCHA is not defined !'
            STOP
         ENDIF
         CALL BEDINI(BEDCHA,MAXBED,TMP)
      ELSE
         DO 20,I=NBED+1,MAXBED
            BEDCHA(I,1)=0
            BEDCHA(I,2)=0
            BEDCHA(I,3)=0
20       CONTINUE
      ENDIF

      IF(.NOT.CHECKBEDCHA(BEDCHA,MAXBED)) THEN
         WRITE(*,'(//A//)')'BEDCHA is incorrect !'
         WRITE(*,*)'NBED = ',NBED
         WRITE(*,'(I3,3F16.8)')(I,BEDCHA(I,1),BEDCHA(I,2),
     &                          BEDCHA(I,3),I=1,NBED)
         STOP
      ENDIF

C Open output files and write headers
C For all values of MODE, 3 output files are produced:
C     *.tab with the numerical output (for matlab)
C     *.dat similar with a header row (for excel)
C     *.txt text output, used if default parameters are modified
      OUTFILE1=OUTFILE(1:LASTCH(OUTFILE))//'.tab'
      OUTFILE2=OUTFILE(1:LASTCH(OUTFILE))//'.dat'
      OUTFILE5=OUTFILE(1:LASTCH(OUTFILE))//'.txt'
      OPEN (11,FILE=OUTFILE1,ERR=92)
      OPEN (12,FILE=OUTFILE2,ERR=93)
      OPEN (15,FILE=OUTFILE5,ERR=96)
      IF(MODE.EQ.14)THEN
         DO 25,i=1,lastch(line1)
            IF(CHAR(9).EQ.LINE1(I:I)) LINE1(I:I)=' '
25       CONTINUE
         LINE1=LINE1(1:LASTCH(LINE1)) //
     &      ' (corrected for diffusion from Sea Carousel)'
      ENDIF
      WRITE(12,'(A)')LINE1(1:LASTCH(LINE1))   ! write first line of input file (comment line)
C write header
      WRITE(CH,100)(I,I=1,21),(I,I=1,20),(I,I=1,20),(I,I=1,20)
100   FORMAT('time Error Tau0eff TauCWS z dz edrate tconc nbconc',
     &9(' c',I1),12(' c',I2),' maxbed',9(' z',I1),11(' z',I2),
     &9(' Te',I1),11(' Te',I2),9(' r',I1),11(' r',I2),' SSCmeasured')
      CALL LINETAB(CH)
      WRITE(12,'(A)')CH(1:LASTCH(CH))

C If MODE<10 then two additional files are produced
C     *.da1 corresponding to the *.dat file of the batch version
C     *.da2 corresponding to the *.da2 file of the batch version
      IF(MODE.LT.10)THEN
         OUTFILE3=OUTFILE(1:LASTCH(OUTFILE))//'.da1'
         OUTFILE4=OUTFILE(1:LASTCH(OUTFILE))//'.da2'
         OPEN (13,FILE=OUTFILE3,ERR=94)
         OPEN (14,FILE=OUTFILE4,ERR=95)
         WRITE(CH,120)
         CALL LINETAB(CH)
         WRITE (13,'(A)')CH(1:LASTCH(CH))
120   FORMAT('DT ERR UB FCW UA U100 USTCWS USTCW Z0 ',
     &'FALL RHEIGHT RLENGTH C0A QS QSDIR SEDM SED SEDDIR')
         WRITE(CH,130)
         CALL LINETAB(CH)
         WRITE (14,'(A)')CH(1:LASTCH(CH))
130   FORMAT('IRUN ',
     &'AB WL USTCRB USTCRS USTUP Z0C PHIB PHI100 DELTACW USTCS',
     &' USTWS USTCWSE USTCWSB USTC USTW RPLCOEF TB1 TB2 TS1 TS2',
     &' PERBED PERSUSP C0 TAOCWS RHOW VISC')
      ENDIF

      RETURN

90    CLOSE(10)
      CLOSE(11)
      CLOSE(12)
      CLOSE(13)
      CLOSE(14)
      STOP
91    WRITE(*,101) INFILE(1:LASTCH(INFILE))
      WRITE(*,102)
      GOTO 90
92    WRITE(*,101) OUTFILE1(1:LASTCH(OUTFILE1))
      GOTO 90
93    WRITE(*,101) OUTFILE2(1:LASTCH(OUTFILE2))
      GOTO 90
94    WRITE(*,101) OUTFILE2(1:LASTCH(OUTFILE3))
      GOTO 90
95    WRITE(*,101) OUTFILE2(1:LASTCH(OUTFILE4))
      GOTO 90
96    WRITE(*,101) OUTFILE2(1:LASTCH(OUTFILE5))
      GOTO 90
101   FORMAT('Error during opening of the file ',A)
102   FORMAT(/,'For explanation of command-line syntax, use : ',
     &'"sedi1d -?"')
      END      ! end of subroutine OPENFILES



C correction of diffusion out of the Sea Carousel
      SUBROUTINE SCARDIFF(TCONC,CONC,TIMEDR,SSCMES,OBS2,ROT)
      IMPLICIT NONE
      INCLUDE 'sed_param.h'
      DOUBLE PRECISION TCONC,CONC(NBCONC)
      DOUBLE PRECISION TIMEDR !Duration of call to Sedtrans (s)
      DOUBLE PRECISION SSCMES ! measured SSC, read from input file (Pa)
      DOUBLE PRECISION OBS2   ! SSC outside of the flume measured by OBS2 (kg/m3)
      DOUBLE PRECISION ROT    ! rotation speed of lid (m/s)

      DOUBLE PRECISION RDIFF  ! relative conc deplation due to diffusion
      DOUBLE PRECISION DIFF   ! diffusion outside of Sea Carousel during time step (kg/m3)
      DOUBLE PRECISION TDIFF  ! total diffusion outside of Sea Carousel so far (kg/m3)
      INTEGER J

      IF(TCONC.GT.0) THEN
         ! decrease CONC for diffusion out of the flume
         DIFF=(TCONC-OBS2)*TIMEDR*0.574d0*ROT/400d0
         RDIFF=(TCONC-DIFF)/TCONC
         DO 55, J=1,NBCONC
            CONC(J)=CONC(J)*RDIFF
55       CONTINUE
      ! remove the previous diffusion correction from the measured SSC (obs1+3)
         IF(ROT.LE.0.01)THEN
            TDIFF=0d0
         ELSE
            SSCMES=SSCMES-TDIFF
            DIFF=(SSCMES-OBS2)*TIMEDR*0.574d0*ROT/400d0
            TDIFF=TDIFF+DIFF
            SSCMES=SSCMES-DIFF
         ENDIF
      ENDIF
      END


C ======================================================================
C ======================================================================
C  For MODE = 10-14, read one input line and check the values, return
C  ERR=0 if successful, ERR>0 if error in input values, ERR=-1 if end of file
C
      SUBROUTINE INPUT1(MODE,ERR,DT,TAUCWS,SSCMES,OBS2,ROT)
      IMPLICIT NONE
      INTEGER MODE,ERR(20),NBERR
      INTEGER DT              ! duration in seconds of the hydrodynamic condtions
      DOUBLE PRECISION TAUCWS ! bed shear stress read from input file (Pa)
      DOUBLE PRECISION SSCMES ! measured SSC, read from input file (Pa)
      DOUBLE PRECISION OBS2   ! SSC outside of the flume measured by OBS2 (kg/m3)
      DOUBLE PRECISION ROT    ! rotation speed of lid (m/s)

50    IF(MODE.EQ.10 .OR. MODE.EQ.11) THEN
         READ(10,*,ERR=50,END=90)DT,TAUCWS
      ELSEIF(MODE.EQ.12) THEN
         READ(10,*,ERR=50,END=90)DT,TAUCWS,SSCMES
      ELSEIF(MODE.EQ.14) THEN
         READ(10,*,ERR=50,END=90)DT,TAUCWS,SSCMES,OBS2,ROT
      ENDIF

      DO 60,NBERR=1,20
60    ERR(NBERR)=0
      NBERR=0
      IF(DT.LE.0 .OR. (DT.GT.60 .AND. MOD(DT,60).NE.0) ) THEN
         NBERR=NBERR+1
         ERR(NBERR)=20
      ENDIF
      IF(TAUCWS.LT.0)THEN
         NBERR=NBERR+1
         ERR(NBERR)=21
      ENDIF
      IF(MODE.EQ.14)THEN
         IF(SSCMES.LT.0.OR.OBS2.GT.100d0)THEN
            NBERR=NBERR+1
            ERR(NBERR)=22
         ENDIF
         IF(OBS2.LT.0.OR.OBS2.GT.100d0)THEN
            NBERR=NBERR+1
            ERR(NBERR)=23
         ENDIF
         IF(ROT.LT.0)THEN
            NBERR=NBERR+1
            ERR(NBERR)=24
         ENDIF
      ENDIF
      RETURN

90    ERR(1)=-1
      END


C ======================================================================
C ======================================================================
C  For all MODE : write cohesive output
C
      SUBROUTINE OUTPUT1(TIME,ERR,TAUCWS,Z,ZS,EDRATE,TCONC,
     &                     CONC,MAXBED,BEDCHA,SSCMES)
      IMPLICIT NONE
      INCLUDE 'sed_param.h'
      INTEGER TIME,ERR(20),MAXBED,I
      DOUBLE PRECISION TAUCWS,Z,ZS,EDRATE,TCONC,SSCMES
      DOUBLE PRECISION CONC(NBCONC),BEDCHA(MAXBED,3)
      CHARACTER*2000 CH       ! used for formatting output
      INTEGER LASTCH          ! function

      IF(ERR(1).NE.0)THEN
         WRITE(CH,*)0.0,ERR(1),
     &                  (0.0,I=1,NBCONC+3*MAXBED+9)
      ELSE
         WRITE(CH,*)TIME,ERR(1),TAU0EFF,TAUCWS,Z,ZS,EDRATE,TCONC,
     &                  NBCONC,CONC,MAXBED,BEDCHA,SSCMES
      ENDIF
      CALL LINETAB(CH)
      WRITE(11,'(A)')CH(1:LASTCH(CH))
      WRITE(12,'(A)')CH(1:LASTCH(CH))
      END   ! end of OUTPUT1



C ======================================================================
C ======================================================================
C  For MODE < 10, write output to the different files
C
      SUBROUTINE OUTPUT2(TIME,ERR,USTCWS,UB,FCW,UA,U100,
     &USTCW,Z0,RHEIGHT,RLENGTH,QS,QSDIR,SEDM,
     &SED,SEDDIR,FALL,C0A,RHOW,VISC,USTCRB,USTCRS,USTUP)
      IMPLICIT NONE
      INCLUDE "sed_param.h" ! declaration of the COMMON blocks
      INTEGER TIME,ERR(20),I
      DOUBLE PRECISION USTCWS,
     &UB,FCW,UA,U100,USTCW,Z0,RHEIGHT,RLENGTH,QS,QSDIR,SEDM,
     &SED,SEDDIR,FALL,C0A,RHOW,VISC,
     @USTCRB,USTCRS,USTUP
      CHARACTER*2000 CH       ! used for formatting output
      INTEGER LASTCH          ! function

      IF (ERR(1).NE.0 .AND. ERR(1).LT.100) THEN
         WRITE(CH,*)TIME,ERR(1),(0,I=1,16)
         CALL LINETAB(CH)
         WRITE(13,'(A)')CH(1:LASTCH(CH)) ! *.da1 file                   ! vers 1.04
         WRITE(CH,*)TIME,(0,I=1,26)
         CALL LINETAB(CH)
         WRITE(14,'(A)')CH(1:LASTCH(CH)) ! *.da2 file                   ! vers 1.04
      ELSE
C write the tabulated output to the *.da1 file
         WRITE(CH,*)TIME,ERR(1),UB,FCW,UA,U100,USTCWS,USTCW,Z0,
     &   FALL,RHEIGHT,RLENGTH,C0A,QS,QSDIR,SEDM,SED,SEDDIR
         CALL LINETAB(CH)
         WRITE(13,'(A)')CH(1:LASTCH(CH))                                ! vers 1.04
C write the additional tabulated output to the *.da2 file
         WRITE(CH,*)TIME,
     &   AB,WL,USTCRB,USTCRS,USTUP,Z0C,PHIB,PHI100,DELTACW,USTCS,
     &   USTWS,USTCWSE,USTCWSB,USTC,USTW,RPLCOEF,TB1,TB2,TS1,TS2,
     &   PERBED,PERSUSP,C0,TAOCWS,RHOW,VISC
         CALL LINETAB(CH)
         WRITE(14,'(A)')CH(1:LASTCH(CH))                                ! vers 1.04
      ENDIF
      END   ! end of OUTPUT2



C ======================================================================
C ======================================================================
C  SUBROUTINE CHECKIN
C
C Checks the validity of the input arguments and reports an error/warning
C code ERR and a message ERRTXT for the first error/warning observed.
C First checks the range of the input arguments, then whether the grain-size
C is adapted for the selected equation. Returns after the first
C observed error/warning, this mean several errors/warning may exist,
C but only the first is reported.
C ERR = 0    --> no error/warning
C ERR = 1-99 --> error, SEDTRANS05 should not be called
C ERR > 100  --> warning, SEDTRANS05 can be called, but warning should be reported

C List of possible errors/warnings
C 1   Incorrect input argument D
C 2   Incorrect input argument UZ
C 3   Incorrect input argument Z
C 4   Incorrect input argument CDIR
C 5   Incorrect input argument HT
C 6   Incorrect input argument PER
C 7   Incorrect input argument WDIR
C 8   Incorrect input argument GD
C 9   Incorrect input argument RHINP
C 10  Incorrect input argument RLINP
C 11  Incorrect input argument BETA
C 12  Incorrect input argument SALIN
C 13  Incorrect input argument RHOS
C 14  Incorrect input argument AULVA
C 15  Incorrect value for NBED
C 16  Incorrect input argument TIMEDR
C 17  Incorrect input argument BEDCHA
C 18  Incorrect input argument CONC
C 19  Incorrect input argument TEMP
C 30  Incorrect input argument IOPT1
C 101 ENGELUND-HANSEN formula not recommended for use with sediments finer than 0.15 mm
C 102 EINSTEIN-BROWN formula is based on laboratory experiments using sediments with grain sizes of 0.3 to 28.6 mm
C 103 BAGNOLD formula is based on laboratory tests with grain sizes between 0.18 and 0.45 mm
C 104 YALIN formula is not recommended for use with sediments smaller than 0.2mm, based on the results of sensitivity analyses
C 105 ACKERS-WHITE formula is based on laboratory experiments using sediments with grain sizes of 0.05 to 29.1 mm
C 107 The cohesive algorithm is not intended for non-cohesive sediments

C IOPT1 = SEDIMENT TRANSPORT PREDICTOR OPTION NUMBER
C            1 - ENGELUND-HANSEN (1967) TOTAL LOAD EQUATION
C            2 - EINSTEIN-BROWN (1950) BEDLOAD EQUATION
C            3 - BAGNOLD (1963) TOTAL LOAD EQUATION
C            4 - YALIN (1963) BEDLOAD EQUATION
C            5 - VAN RIJN (1993) BEDLOAD EQUATION
C            7 - Cohesive sediments algorithm

      SUBROUTINE CHECKIN(ERR,D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,
     &RLINP,BETA,RHOS,SALIN,TEMP,IOPT1,TIMEDR,AULVA,
     &BEDCHA,MAXBED,CONC)
      IMPLICIT NONE
C **************** input variables **********************
      DOUBLE PRECISION D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,RLINP,BETA,
     @RHOS,SALIN,TEMP,TIMEDR,AULVA
      INTEGER IOPT1
      INTEGER MAXBED          ! upper bound of first dimension of BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
      INTEGER NBCONC
      PARAMETER (NBCONC = 21) ! number of element in CONC
      DOUBLE PRECISION CONC(NBCONC) !array of suspended sediment concentration (mg/l)
C **************** output variables **********************
      INTEGER ERR(20)          ! Error/Warning code
C **************** intermediate variables **********************
      DOUBLE PRECISION TCONC
      LOGICAL CHECKBEDCHA   ! function
      INTEGER I,NB

      DO 10,NB=1,20
10    ERR(NB)=0
      NB=1

C Errors for input argument outside of accetable range
      IF (D.LE.0.) THEN
         ERR(NB)=1
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (UZ.LT.0.) THEN
         ERR(NB)=2
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (Z.LE.0. .OR. Z.GT.D) THEN
         ERR(NB)=3
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (CDIR.LT.0. .OR. CDIR.GT.360.) THEN
         ERR(NB)=4
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (HT.LT.0 .OR. HT.GE.(2d0*D)) THEN
         ERR(NB)=5
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (HT.GT.0) THEN
         IF (PER.LT.0.) THEN
            ERR(NB)=6
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
         IF (WDIR.LT.0. .OR. WDIR.GT.360.) THEN
            ERR(NB)=7
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
      ENDIF
      IF (GD.LT.0. .OR. GD.GT.0.256) THEN
         ERR(NB)=8
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (RHINP.GE.D .OR. RHINP.LT.0.) THEN
         ERR(NB)=9
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (RLINP.LT.0. .OR. (RLINP.EQ.0. .AND. RHINP.GT.0.)) THEN
         ERR(NB)=10
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (BETA.LT.-35. .OR. BETA.GT.35.) THEN
         ERR(NB)=11
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (SALIN.LT.0. .OR. SALIN.GT.42.) THEN
         ERR(NB)=12
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (RHOS.LE.1025. .OR. RHOS.GE.22000.) THEN
         ERR(NB)=13
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (IOPT1.EQ.7) THEN
         IF (AULVA.LT.0. .OR. AULVA.GT.1.) THEN
            ERR(NB)=14
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
         IF (TIMEDR.LE.0.) THEN
            ERR(NB)=16
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
         IF (.NOT.CHECKBEDCHA(BEDCHA,MAXBED)) THEN
            ERR(NB)=17
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
         TCONC=0.0d0
         DO 20, I=1,NBCONC
            IF (CONC(I).LT.0) GOTO 30
            TCONC=TCONC+CONC(I)
20       CONTINUE
         IF (TCONC.LE.1500) GOTO 40
30          ERR(NB)=18
            NB=NB+1
         IF(NB.GT.20) RETURN
40       CONTINUE
      ENDIF
      IF (TEMP.LE.0. .OR. TEMP.GT.35.) THEN
         ERR(NB)=19
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (IOPT1.LT.1 .OR. IOPT1.GT.7 .OR. IOPT1.EQ.6) THEN
         ERR(NB)=30
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF

C Warnings for grain-size not adapted for selected formula
      GOTO(210,220,230,240,250,500,270)IOPT1
 210  IF (GD.LT.0.00015) THEN
         ERR(NB)=101
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 220  IF (GD .LT. 0.0003 .OR. GD .GT. .0286) THEN
         ERR(NB)=102
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 230  IF (GD.LT.0.00018 .OR. GD.GT.0.00045)THEN
         ERR(NB)=103
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 240  IF (GD.LT.0.0002)THEN
         ERR(NB)=104
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 250  IF (GD.LT.0.0002 .OR. GD.GT.0.002)THEN
         ERR(NB)=105
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 270  IF (GD.GT.0.00006)THEN
C         ERR(NB)=107
C         NB=NB+1
C         IF(NB.GT.20) RETURN
      ENDIF

 500  CONTINUE

      END    ! end of subroutine CHECKIN



C ======================================================================
C ======================================================================
C  SUBROUTINE ERRTEXT
C
C Return the text corresponding to the error code, which was produce by CHECKIN
C
      SUBROUTINE ERRTEXT(ERR,ERRTXT)
      INTEGER ERR          ! Error/Warning code
      CHARACTER*140 ERRTXT ! Error/Warning text corresponding to ERR, if the
                           ! text is longer than 75 characters, ERRTXT(1:75) is
                           ! the 1st line and ERRTXT(76:140) is the second line
      CHARACTER STXT*25
      PARAMETER (STXT='Incorrect input argument ')

      ERRTXT=' '
      IF(ERR.GE.1 .AND. ERR.LE.24) THEN
         GOTO (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,
     &            20,21,22,23,24),ERR
1        ERRTXT=STXT // 'D'
         GOTO 500
2        ERRTXT=STXT // 'UZ'
         GOTO 500
3        ERRTXT=STXT // 'Z'
         GOTO 500
4        ERRTXT=STXT // 'CDIR'
         GOTO 500
5        ERRTXT=STXT // 'HT'
         GOTO 500
6        ERRTXT=STXT // 'PER'
         GOTO 500
7        ERRTXT=STXT // 'WDIR'
         GOTO 500
8        ERRTXT=STXT // 'GD'
         GOTO 500
9        ERRTXT=STXT // 'RHINP'
         GOTO 500
10       ERRTXT=STXT // 'RLINP'
         GOTO 500
11       ERRTXT=STXT // 'BETA'
         GOTO 500
12       ERRTXT=STXT // 'RHOW'
         GOTO 500
13       ERRTXT=STXT // 'RHOS'
         GOTO 500
14       ERRTXT=STXT // 'AULVA'
         GOTO 500
15       ERRTXT='Incorrect value for NBED'
         GOTO 500
16       ERRTXT=STXT // 'TIMEDR'
         GOTO 500
17       ERRTXT=STXT // 'BEDCHA'
         GOTO 500
18       ERRTXT=STXT // 'CONC'
         GOTO 500
19       ERRTXT=STXT // 'TEMP'
         GOTO 500
20       ERRTXT=STXT // 'DT'
         GOTO 500
21       ERRTXT=STXT // 'TAUCWS'
         GOTO 500
22       ERRTXT=STXT // 'SSCMES'
         GOTO 500
23       ERRTXT=STXT // 'OBS2'
         GOTO 500
24       ERRTXT=STXT // 'ROT'
         GOTO 500

      ELSEIF(ERR.EQ.30) THEN
         ERRTXT=STXT // 'IOPT1'
         GOTO 500
      ELSEIF(ERR.GE.101 .AND. ERR.LE.107) THEN
         GOTO(101,102,103,104,105,500,107), (ERR-100)
101      ERRTXT(1:75)='ENGELUND-HANSEN formula not recommended for '//
     &      'use with sediments finer than'
         ERRTXT(76:140)='0.15 mm'
         GOTO 500
102      ERRTXT(1:75)='EINSTEIN-BROWN formula is based on laboratory '//
     &      'experiments using sediments'
         ERRTXT(76:140)='with grain sizes of 0.3 to 28.6 mm'
         GOTO 500
103      ERRTXT(1:75)='BAGNOLD formula is based on laboratory tests '//
     &      'with grain sizes between 0.18'
         ERRTXT(76:140)='and 0.45 mm'
         GOTO 500
104      ERRTXT(1:75)='YALIN formula is not recommended for use with '//
     &      'sediments smaller than 0.2mm,'
         ERRTXT(76:140)='based on the results of sensitivity analyses'
         GOTO 500
105      ERRTXT(1:75)='VAN RIJN formula is based on laboratory '//
     &      'experiments using sediments'
         ERRTXT(76:140)='with grain sizes of 0.2 to 2.0 mm'
         GOTO 500
107      ERRTXT='The cohesive algorithm is not intended for '//
     &      'non-cohesive sediments'
         GOTO 500
      ENDIF
500   CONTINUE
      END   ! end of subroutine ERRTEXT


      INCLUDE 'cohesive.f'
      INCLUDE 'sedtrans-core.f'

