C Sedtrans05 - a sediment transport model
C Copyright (C) 2005 Urs Neumeier, Christian Ferrarin, Carl Amos
C and Georg Umgiesser.
C Sedtrans05 was developed at the ISMAR-CNR in Venice and the
C University of Southampton.
C Previous versions of Sedtrans (1992 and 1996) were developed at
C the Geological Survey of Canada, Bedford Institute of Oceanography.
C
C This program is free software; you can redistribute it and/or modify
C it under the terms of the GNU General Public License as published by
C the Free Software Foundation; either version 3 of the License, or
C (at your option) any later version. See file "licence.txt" in the
C Sedtrans05 directory for the full licence text, which can also be
C found at http://www.gnu.org/licenses/
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C **********************************************************************
C This is the main file of the single-point version of SEDTRANS05,
C interactive mode and batch mode. All other source-files are called with
C INCLUDE statements.
C
C
C Start the program with the command line option '-?' to see the syntax
C of the command line options/arguments
C
      PROGRAM SED05
      IMPLICIT NONE
      
      CHARACTER VERSION*12 /'version 1.04'/  ! version of the program
      LOGICAL BATCH            ! True for batch mode (read input from file)
      LOGICAL FHEADER          ! True if header line in output files
      LOGICAL ONEFILE          ! True if 1 instead of 3 tabular output files
      LOGICAL INSALTEMP        ! True if input is Temp./Salinity, otherwise water density/dynam.viscosity
      LOGICAL OUTCOH           ! True if file for cohesive output is already opened
      CHARACTER INFILE*200 /' '/! Name of input file (batch mode)
      CHARACTER OUTFILE*200/' '/! Name of output file
      CHARACTER CONSTFILE*200/' '/! Name of file to modify the default parameters
      INTEGER ABORT            ! Returned from READIN,
                               !      0: normal input
                               !      1: input aborted or critical error found
                               !      2: end of file
      INTEGER ERR(20)          ! Error/Warning code (from CHECKIN)
      CHARACTER CH*10
      INTEGER IRUN /0/         ! ID for each run
      DOUBLE PRECISION TEMP    ! Temperature C
      DOUBLE PRECISION SALIN   ! Salinity
      CHARACTER*80 BEDTXT(7)   ! Text of bedform prediction

      INCLUDE "sed_param.h" ! declaration of the COMMON blocks

C ****** input variables to SEDTRANS05
      INTEGER IOPT1
      DOUBLE PRECISION D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,RLINP,
     @BETA,RHOS,RHOW,VISC,TIMEDR,AULVA
      INTEGER MAXBED
      PARAMETER (MAXBED=20)
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
      DOUBLE PRECISION CONC(NBCONC) !array of suspended sediment concentration (mg/l)
C ****** output variables from SEDTRANS05
      DOUBLE PRECISION
     &UB,FCW,UA,U100,USTCWS,USTCW,Z0,RHEIGHT,RLENGTH,QS,QSDIR,SEDM,
     @SED,SEDDIR,FALL,C0A,EDRATE,ZS,TAOS,TCONC,
     @USTCRB,USTCRS,USTUP

C If the functions retrieving the command-line arguments are not suported
C by the compiler, change the following lines, comment the CALL COMARG line
C and remove the subroutine COMARG, READCOMARG and the function NBCOMARG
C      BATCH=.TRUE.  ! for batch mode
      BATCH=.FALSE. ! for interactive input (default)
      FHEADER=.TRUE.   ! for header lines in output files (default)
C      FHEADER=.FALSE.  ! for header lines in output files
C      ONEFILE=.TRUE. ! for 1 tabular output file (*.dat)
      ONEFILE=.FALSE. ! for 3 tabular output files (*.dat, *.da2, *.pro), default
      INSALTEMP=.TRUE.   ! input is Salinity / Temperature
C      INSALTEMP=.FALSE.  ! input is water density and dynamic viscosity
      CALL COMARG(BATCH,FHEADER,ONEFILE,INSALTEMP,INFILE,
     &            OUTFILE,CONSTFILE,VERSION)  ! read the command line argumentS

      CALL INICONST! initialize the constants in the common blocks

      CALL OPENFILE(BATCH,FHEADER,ONEFILE,INFILE,OUTFILE,CONSTFILE,
     &              OUTCOH,VERSION)    ! open input and output files

      TEMP=-999.  ! set TEMP to indicate first run

C MAIN LOOP
10    CONTINUE

      CALL READIN(BATCH,INSALTEMP,ABORT,ERR,   ! read the input paramters
     &IRUN,IOPT1,D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,RLINP,
     &BETA,RHOS,SALIN,TEMP,BEDCHA,MAXBED,CONC,TIMEDR,AULVA)

      IF (ABORT.EQ.0) THEN

         IF (INSALTEMP) THEN
C compute the dynamic/molecular viscosity
            CALL DENSVISC(TEMP,SALIN,RHOW,VISC)
         ELSE
            RHOW=TEMP
            VISC=SALIN
         ENDIF

         RHEIGHT=RHINP
         RLENGTH=RLINP
         CALL SEDTRANS05(            ! call to the core-part of Sedtrans
     @D,UZ,Z,CDIR,HT,PER,WDIR,GD,
     @BETA,RHOS,RHOW,IOPT1,VISC,CONC,BEDCHA,MAXBED,TIMEDR,AULVA, !**INPUT VARIABLES**                                					   	!**INPUT VARIABLES**
     @
     @USTCRB,USTCRS,USTUP,UB,FCW,UA,U100,USTCWS,USTCW,Z0,RHEIGHT,
     @RLENGTH,QS,QSDIR,SEDM,SED,SEDDIR,FALL,C0A,EDRATE,ZS,TAOS,
     @TCONC)		!**OUTPUT VARIABLES**

         CALL BEDFORM(U100,UA,UB,GD,FCW,RHOW,RHOS,VISC,IOPT1,
     @RHEIGHT,RLENGTH,USTCWS,USTCRB,USTCRS,USTUP,BEDTXT)    ! predict the bedforms

      ENDIF

      IF (ABORT.EQ.0 .OR. (ABORT.EQ.1 .AND. ERR(1).NE.0) ) THEN
         CALL WRITEOUT(BATCH,FHEADER,ONEFILE,OUTCOH,OUTFILE,ERR,
     &BEDTXT,IRUN,IOPT1,CONC,BEDCHA,MAXBED,RHOW,
     &UB,FCW,UA,U100,USTCWS,USTCW,Z0,RHEIGHT,RLENGTH,QS,QSDIR,SEDM,
     @SED,SEDDIR,FALL,C0A,EDRATE,ZS,TAOS,TCONC,VISC,
     @USTCRB,USTCRS,USTUP)  ! write the results (screen/output files)
      ENDIF
      
      IF (.NOT.BATCH) THEN
         WRITE(*,101)
101      FORMAT(/'Do you want to do another run ? (Y/N)')
         READ(*,*)CH
         IF(CH.NE.'Y' .AND. CH.NE.'y' .AND. CH.NE.'YES' .AND.
     &         CH.NE.'yes') ABORT=2
      ENDIF
      
C MAIN LOOP
      IF (ABORT.NE.2) GOTO 10

      IF (BATCH) CLOSE (10)
      CLOSE (11)
      CLOSE (12)
      CLOSE (13)
      CLOSE (14)
      IF (OUTCOH) CLOSE (15)

      END    ! end of main PROGRAM
      



C ======================================================================
C ======================================================================
C SUBROUTINE READCOMARG
C
C Read one argument of the the command line.
C
C This routine uses a non stantard functions to retrieve the argument
C from the command line:
C CALL GETARG(n,argument_number_n)
C
C IF THIS PROGRAM IS COMPILED WITH ANOTHER COMPILER THAN G77, IT IS LIKELY
C THAT THIS MUST BE MODIFIED !
      SUBROUTINE READCOMARG (N, ARGUMENT)
      IMPLICIT NONE
      INTEGER N              ! number of arguments or position or argument
      CHARACTER ARGUMENT*(*) ! argument that is returned
      INTEGER NBCOMARG          ! function
      INTEGER NBARG          ! temporary variable
      
      NBARG=NBCOMARG()
      IF ((N.GE.1) .AND. (N.LE.NBARG)) THEN
         CALL GETARG(N,ARGUMENT)
      ENDIF
      END      ! end of subroutine READCOMARG



C ======================================================================
C ======================================================================
C FUNCTION NBCOMARG
C
C Return the number of command-line arguments.
C
C This routine uses a non stantard functions to retrieve the argument
C from the command line:
C nb_argument = IARGC()
C
C IF THIS PROGRAM IS COMPILED WITH ANOTHER COMPILER THAN G77, IT IS LIKELY
C THAT THIS MUST BE MODIFIED !
      INTEGER FUNCTION NBCOMARG ()
      IMPLICIT NONE
      INTEGER IARGC          ! function
      NBCOMARG=IARGC()
      END      ! end of function READCOMARG




C ======================================================================
C ======================================================================
C SUBROUTINE COMARG
C
C Read the arguments on the command line.
C Return BATCH=.TRUE. (with INFILE and OUTFILE when given) if the
C batch mode is requested.
C Print instructions on the screen if command-line argument is -? , then stop
C Generate an example input-file "INDATA.DAT" if command line argument
C is -e, then stop.
C Print short error message when command-line argument is not recognized, then stop.
C Return BATCH=.FALSE. if no command-line argument are given.

      SUBROUTINE COMARG(BATCH,FHEADER,ONEFILE,INSALTEMP,INFILE,
     &            OUTFILE,CONSTFILE,VERSION)
      
      IMPLICIT NONE
      LOGICAL BATCH        ! True for batch mode (read input from file)
      LOGICAL FHEADER      ! True if header line in output files
      LOGICAL ONEFILE      ! True if 1 instead of 3 tabular output files
      LOGICAL INSALTEMP    ! True if input is Temp./Salinity, otherwise water density/dynam.viscosity
      CHARACTER INFILE*(*) ! Name of input file (batch mode)
      CHARACTER OUTFILE*(*)! Name of output file
      CHARACTER CONSTFILE*(*)! Name of file to modify the default parameters
      CHARACTER VERSION*(*)! version of the program

      CHARACTER PROGNAME*5/'SED05'/
      INTEGER NBARG        ! Number of command-line argument
      CHARACTER ARG*200    ! command-line argument
      INTEGER I,POS,LASTCH
      INTEGER NBFILES      ! Number of processed filenames
      INTEGER NBCOMARG        ! function

C This routine uses two non stantard functions to retrieve the argument
C from the command line:
C nb_argument = IARGC()
C CALL GETARG(n,argument_number_n)

      NBARG=NBCOMARG()
      IF (NBARG.EQ.0) THEN
         RETURN
      ENDIF

      NBFILES=0
      POS=1
10    CONTINUE     ! beginning of loop
      CALL READCOMARG(POS,ARG)
      IF (POS.EQ.1) THEN          ! process first the special options
         IF (ARG.EQ.'-?' .OR. ARG.EQ.'/?') THEN
            WRITE(*,100)VERSION,(PROGNAME,I=1,6)
            STOP
         ELSEIF (ARG.EQ.'-l' .OR. ARG.EQ.'-L') THEN
            WRITE(*,701)VERSION
            WRITE(*,702)
            STOP
         ELSEIF (ARG.EQ.'-E' .OR. ARG.EQ.'-e' .OR.
     &         ARG.EQ.'-EC' .OR. ARG.EQ.'-ec') THEN
            CALL EXAMPLE(VERSION)     ! generate example files
            CALL SETCCONST(' ',0,' ')  ! generate example file to modify default parameters
            STOP
         ENDIF
      ENDIF
      IF (ARG.EQ.'-B' .OR. ARG.EQ.'-b') THEN   ! batch mode
         BATCH=.TRUE.
         IF (NBARG.GT.POS) THEN
            CALL READCOMARG(POS+1,ARG)
            IF(ARG(1:1).NE.'-')THEN
               INFILE=ARG
               NBFILES=NBFILES+1
               POS=POS+1
               IF (NBARG.GT.POS) THEN
                  CALL READCOMARG(POS+1,ARG)
                  IF(ARG(1:1).NE.'-')THEN
                     OUTFILE=ARG
                     NBFILES=NBFILES+2
                     POS=POS+1
                  ENDIF
               ENDIF
            ENDIF
         ENDIF
      ELSEIF (ARG.EQ.'-O' .OR. ARG.EQ.'-o') THEN   ! output file
         IF (NBARG.GT.POS) THEN
            CALL READCOMARG(POS+1,ARG)
            IF(ARG(1:1).NE.'-')THEN
               OUTFILE=ARG
               NBFILES=NBFILES+2
               POS=POS+1
            ELSE
               WRITE(*,102)        ! incorrect usage
               STOP
            ENDIF
         ELSE
            WRITE(*,102)        ! incorrect usage
            STOP
         ENDIF
      ELSEIF (ARG.EQ.'-C' .OR. ARG.EQ.'-c') THEN   ! default parameter file
         IF (NBARG.GT.POS) THEN
            CALL READCOMARG(POS+1,ARG)
            IF(ARG(1:1).NE.'-')THEN
               CONSTFILE=ARG
               POS=POS+1
            ELSE
               WRITE(*,103)     ! incorrect usage
               STOP
            ENDIF
         ELSE
            WRITE(*,103)        ! incorrect usage
            STOP
         ENDIF
      ELSEIF (ARG.EQ.'-H' .OR. ARG.EQ.'-h') THEN   ! option headerline in output files
         FHEADER=.FALSE.

      ELSEIF (ARG.EQ.'-F' .OR. ARG.EQ.'-f') THEN   ! option one output file
         ONEFILE=.TRUE.
      ELSEIF (ARG.EQ.'-V' .OR. ARG.EQ.'-v') THEN   ! option input density/viscosity
         INSALTEMP=.FALSE.
      ELSE
         WRITE(*,101)ARG(1:LASTCH(ARG)) ! unrecognized option
         STOP
      ENDIF
      POS=POS+1
      IF(POS.LE.NBARG) GOTO 10

C for batch mode, use same input filename for output if no name conflicts
      IF(BATCH.AND.NBFILES.EQ.1)THEN
         POS=LASTCH(INFILE)
         ARG=INFILE(MAX(1,POS-3):POS)
         DO 20,I=1,4
            IF(ARG(I:I).GE.'A'.AND.ARG(I:I).LE.'Z')
     &         ARG(I:I)=CHAR(ICHAR(ARG(I:I))+32)
20       CONTINUE
         IF(ARG.NE.'.dat' .AND. ARG.NE.'.txt' .AND. ARG.NE.'.coh' .AND.
     &      (ONEFILE .OR. (ARG.NE.'.da2' .AND. ARG.NE.'.pro'))) THEN
            POS=INDEX(INFILE,'.')
            IF(POS.GT.1)THEN
               OUTFILE=INFILE(1:POS-1)
               GOTO 40
            ENDIF
         ENDIF
      ENDIF
40    CONTINUE

100   FORMAT(/,
     &'Command-line syntax for using the 1D version of Sedtrans05 (',
     &A,')',//,
     &A,' [-b [IN-FILE [OUT-FILE]]] [-o OUT-FILE] ',
     &'[-c PARAMFILE] [-f] [-h] [-v]',//
     &T6,' -b',T12,'Batch mode with optionally names of input and ',
     &'output files',/,
     &T12,'(default is interactive mode)',/
     &T6,' -o',T12,'Specify the name of the output files',/,
     &T6,' -c',T12,'Modify defaults parameters according to the ',
     &'file PARAMFILE',/
     &T6,' -f',T12,'Only one tabular output file for non-cohesive ',
     &'sediments',/,
     &T6,' -h',T12,'No header line in tabular output files',/,
     &T6,' -v',T12,'Input water density/dyn.viscosity instead of ',
     &'salinity/temperature',/,
     &'Special options:',/
     &A,' -?',T12,'Show the present help text'/,
     &A,' -L',T12,'Show the licence',/,
     &A,' -e',T12,'Generate an example input-file ',
     &'"INDATA.DAT" for batch-mode',/,
     &T12,'runs, an example file "INDATA.CST" to modify ',
     &'the default ',/,
     &T12,'parameters and write the list of abbreviations ',
     &'used in the',/,
     &T12,'output files to the file "OUTPUT.LST"',//,
     &'IN-FILE must be with extension; default IN-FILE ',
     &'(batch mode) is',/,'"INDATA.DAT". OUT-FILE must be without ',
     &'extension; default OUT-FILE ',/,
     &'(batch and interactive modes) is "OUTPUT.*".',/
     &'Examples of usage:'/,
     &T4,A,'               Interactive mode with default output files',/
     &T4,A,' -b cases.csv  Batch mode with input "cases.csv" and ',
     &'output "cases.*"')

101   FORMAT (/'Unrecognized option "',A,'". For help use option -?'/)
102   FORMAT (/'Incorrect usage of option -o. For help use option -?'/)
103   FORMAT (/'Incorrect usage of option -c. For help use option -?'/)
C The same formats 701 and 702 exist in COMARG and in OPENFILE, they
C must be updated together !
701   FORMAT(
     &/,'SEDTRANS05 :  A SEDIMENT TRANSPORT MODEL (',A,')',/
     &/,'Copyright (C) 2005 Urs Neumeier, Christian Ferrarin, Carl Amos'
     &/,'and Georg Umgiesser.',
     &/,'Sedtrans05 was developed at the ISMAR-CNR in Venice and the',
     &/,'University of Southampton.',
     &/,'Previous versions of Sedtrans (1992 and 1996) were developed',
     &' at',
     &/,'the Geological Survey of Canada, Bedford Institute of ',
     &'Oceanography.',/)
702   FORMAT(
     &'This program is free software; you can redistribute it and/or',
     &' modify',
     &/,'it under the terms of the GNU General Public License as ',
     &'published by',
     &/,'the Free Software Foundation; either version 3 of the ',
     &'License, or',
     &/,'(at your option) any later version. See file "licence.txt" ',
     &'in the'
     &/,'Sedtrans05 directory for the full licence text, which can ',
     &'also be',
     &/,'found at http://www.gnu.org/licenses/',
     &/,
     &/,'This program is distributed in the hope that it will be ',
     &'useful,',
     &/,'but WITHOUT ANY WARRANTY; without even the implied warranty of'
     &/,'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the',
     &/,'GNU General Public License for more details.')

      END      ! end of subroutine COMARG



C ======================================================================
C ======================================================================
C SUBROUTINE EXAMPLE
C
C Generate an input example-file (INDATA.DAT) and a file (OUTDATA.LST)
C with the list of abbreviations used in the output files

      SUBROUTINE EXAMPLE(VERSION)
      IMPLICIT NONE
      INCLUDE "sed_param.h" ! declaration of the COMMON block /WSCLASS/
      CHARACTER NAME*20,CH*500
      CHARACTER VERSION*(*)! version of the program
      LOGICAL EX
      INTEGER I,LASTCH
      CHARACTER*140 ERRTXT ! Error/Warning text corresponding to ERR, if the
                           ! text is longer than 75 characters, ERRTXT(1:75) is
                           ! the 1st line and ERRTXT(76:140) is the second line
      NAME='INDATA.DAT'
      INQUIRE(FILE=NAME, EXIST=EX)
      IF (EX) THEN
         WRITE(*,102) NAME(1:LASTCH(NAME))
         READ(*,*) CH
         IF (CH.NE.'Y' .AND. CH.NE.'y') THEN
            WRITE(*,103) NAME(1:LASTCH(NAME))
            GOTO 10
         ENDIF
      ENDIF
      OPEN(10,FILE=NAME,ERR=20)

      WRITE(10,120,ERR=20)VERSION
      WRITE(CH,121)
      CALL LINETAB(CH)
      WRITE(10,'(A)',ERR=20)CH(1:LASTCH(CH))
      WRITE(CH,122)
      CALL LINETAB(CH)
      WRITE(10,'(A)',ERR=20)CH(1:LASTCH(CH))
      WRITE(10,123,ERR=20)
      WRITE(CH,124)(I,I=1,21),(I,I=1,4),(I,I=1,4),(I,I=1,4)
      CALL LINETAB(CH)
      WRITE(10,'(A)',ERR=20)CH(1:LASTCH(CH))
      WRITE(CH,125)
      CALL LINETAB(CH)
      WRITE(10,'(A)',ERR=20)CH(1:LASTCH(CH))
      WRITE(10,126,ERR=20)
      CLOSE(10,ERR=20)

10    NAME='OUTPUT.LST'
      INQUIRE(FILE=NAME, EXIST=EX)
      IF (EX) THEN
         WRITE(*,102) NAME(1:LASTCH(NAME))
         READ(*,*) CH
         IF (CH.NE.'Y' .AND. CH.NE.'y') THEN
            WRITE(*,106) NAME(1:LASTCH(NAME))
            RETURN
         ENDIF
      ENDIF
      OPEN(10,FILE=NAME,ERR=40)

      WRITE(10,130,ERR=40)VERSION
      WRITE(10,131,ERR=40)
      WRITE(10,132,ERR=40)
      CALL INICONST
      WRITE(10,133,ERR=40)(I,WSI(I),I=1,NBCONC)
      WRITE(10,134,ERR=40)
      DO 15,I=1,107
         IF(I.EQ.100)WRITE(10,135,ERR=40)
         CALL ERRTEXT(I,ERRTXT)
         IF(ERRTXT.NE.' ')THEN
            IF (LASTCH(ERRTXT).LE.75) THEN
               WRITE(10,136,ERR=40)I,ERRTXT(1:LASTCH(ERRTXT))
            ELSE
               WRITE(10,137,ERR=40)I,ERRTXT(1:75),
     &               ERRTXT(76:LASTCH(ERRTXT))
            ENDIF
         ENDIF
15    CONTINUE

      CLOSE(10)
      RETURN

20    WRITE(*,104) NAME(1:LASTCH(NAME))
      CLOSE (10,ERR=30)
30    GOTO 10
40    WRITE(*,104) NAME(1:LASTCH(NAME))
      CLOSE (10,ERR=50)
50    RETURN

102   FORMAT (/'The existing file ',A,' will be overwritten, '/,
     &'do you want to continue ? (Y/N)')
103   FORMAT (/'Generation of example ',A,' aborted.')
104   FORMAT (/'Error during opening/writing file ',A,/)
106   FORMAT (/'Generation of file ',A,' aborted.')
C Example INDATA.DAT file
120   FORMAT (
     &'Example of an input file for the batch mode of Sedtrans05 (',
     &A,')',/,
     &'Comment lines (any line containing non numerical characters) ',/
     &'and empty lines are ignored.',/,
     &'The numerical values in the file must be separated by spaces, ',
     &/,'commas, or tabulators.',/
     &'The input format is different for non-cohesive and cohesive ',/,
     &'sediment. Both input types can be mixed in the same file.',//,
     &'For the non-cohesive equations, the input format is ')
121   FORMAT('IRUN IOPT1 D UZ Z CDIR HT PER WDIR GD RHINP RLINP BETA ',
     &'RHOS SALIN TEMP')
122   FORMAT('1 2 2.2 0.8 0.5 30.5 0.3 3.5 185. 0.00035 0.0 0.0 -2.0 ',
     &'2650. 35. 15.')
123   FORMAT(/,'For cohesive sediment, the input format is' )
124   FORMAT('IRUN IOPT1 D UZ Z CDIR HT PER WDIR GD RHINP RLINP BETA ',
     &'RHOS SALIN TEMP TIMEDR AULVA',9(' c',I1),12(' c',I2),
     &' NBED',4(' z',I1),4(' Tero',I1),4(' rho',I1))
125   FORMAT('2 7 2.2 0.8 0.5 30.5 0.3 3.5 185. 0.0 0.0 0.0 0.0 ',
     &'2650. 35. 15. 300. 0.0',21(' 0.0'),' 4 ',
     &'0.0 0.005 0.02 0.05 ',
     &'0.2 0.3 0.4 0.45 ',
     &'200 300 400 450')
126   FORMAT(/,'Meaning of the abbreviations',/,
     &'IRUN',T10,'identifier for the run (integer value)  #',/,
     &'IOPT1',T10,'selection of sediment transport equation ',
     &            '(1,2,3,4,5 or 7)  #',/,
     &'D',T10,'water depth (m)',/,
     &'UZ',T10,'current speed (m/s)',/,
     &'Z',T10,'height of current measurement (m)',/,
     &'CDIR',T10,'current direction (degrees north)',/,
     &'HT',T10,'wave height (m)',/,
     &'PER',T10,'wave period (seconds)',/,
     &'WDIR',T10,'wave direction (degrees north)',/,
     &'GD',T10,'grain size (m)',/,
     &'RHINP',T10,'ripple height (m)',/,
     &'RLINP',T10,'ripple length (m)',/,
     &'BETA',T10,'bed slope (degree)',/,
     &'RHOS',T10,'grain density (default=2650, kg/m**3)',/,
     &'SALIN',T10,'salinity (default=35)',/,
     &'TEMP',T10,'temperature (degrees Celsius)',/,
     &'TIMEDR',T10,'deposition/erosion duration (seconds)',/,
     &'AULVA',T10,'fraction of bed area covered by Ulva (range 0-1)',/,
     &'c1-c21',T10,'suspended sediment concentration of each Ws class'
     &,/,'NBED',T10,'number (N) of entries in BEDCHA  #',/,
     &'z1-zN',T10,'depth (positive downward, m)',/,
     &'rho1-rhoN  dry bulk density (kg/m**3)',/,
     &'Tero1-TeroN  critical erosion stress (Pa)',/
     &'#  if a real number is given for these parameters, it will be ',
     &'rounded to the nearest integer')
C List of output abbreviations for the file OUTPUT.LST
130   FORMAT('Abbreviations used in the output files of Sedtrans05 ',
     &'(interactive/batch version, ',A,')',/,/,
     &'In the output file *.dat (main results)',/,/,
     &'IRUN',T9,'identifier for the run (integer value)',/,
     &'ERR',T9,'error/warning code (see below)',/,
     &'UB',T9,'maximum wave induced orbital velocity at the bottom ',
     &'(m/s)',/,
     &'FCW',T9,'bottom (skin) friction factor',/,
     &'UA',T9,'current speed to be used in bottom stress calc. (m/sec)'/
     &'U100',T9,'current speed at 1 m. above seabed (m/sec)',/,
     &'USTCWS',T9,'combined skin-friction shear velocity of GM (m/sec)'/
     &'USTCW',T9,'combined total shear velocity of GM (m/sec)',/,
     &'Z0',T9,'bed roughness length (m)',/,
     &'FALL',T9,'settling velocity for non-cohesive sediment (m/sec)',/,
     &'RHEIGHT',T9,'predicted ripple height (m)',/,
     &'RLENGTH',T9,'predicted ripple length (m)',/,
     &'C0A',T9,'depth averaged reference concentration at z0 (kg/m^3)'/
     &'QS',T9,'suspended sediment transport rate (kg/m/s)',/,
     &'QSDIR',T9,'direction of suspended sediment transport (degree)',/,
     &'SEDM',T9,'time-averaged net sediment transport as mass (kg/s/m)'/
     &'SED',T9,'time-averaged net sediment transport as volume ',
     &'(m**3/s/m)',/,
     &'SEDDIR',T9,'direction of net sediment transport ',
     &'(azimuth,degrees)',//)
131   FORMAT('In the output file *.da2 (additional parameters)',/,/,
     &'IRUN',T9,'identifier for the run (integer value)',/,
     &'AB',T9,'excursion length of bottom wave orbit (m)',/,
     &'WL',T9,'wave length (m)',/,
     &'USTCRB',T9,'critical shear vel for initi of bedload trans ',
     &'(m/sec)',/,
     &'USTCRS',T9,'critical shear vel for initi of suspended load ',
     &'transport (m/sec)',/,
     &'USTUP',T9,'critical shear vel for initn of sheet flow ',
     &'transport (m/sec)',/,
     &'Z0C',T9,'apparent bed roughness length (m)',/,
     &'PHIB',T9,'angle between wave and current directions (radians)',/,
     &'PHI100',T9,'angle between wave and current directions at ',
     &'1 m. above seabed (radians)',/,
     &'DELTACW',T9,'height of the wave-current boundary layer (m)',/,
     &'USTCS',T9,'current skin-friction shear velocity of GM (m/sec)',/,
     &'USTWS',T9,'wave skin-friction shear velocity of GM (m/sec)',/,
     &'USTCWSE',T9,'effective combined skin-friction shear velocity ',
     &'(m/sec)',/,
     &'USTCWSB',T9,'transport-related combined shear velocity (m/sec)',/
     &,'USTC',T9,'total current shear velocity of GM (m/sec)',/,
     &'USTW',T9,'total wave shear velocity of GM (m/sec)',/,
     &'RPLCOEF',T9,'ripple coefficient for shear velocity conversion',/,
     &'TB1',T9,'time at which bedload transport ceases (sec)',/,
     &'TB2',T9,'time at which bedload transport recommences (sec)',/,
     &'TS1',T9,'time at which suspended load transport ceases (sec)',/,
     &'TS2',T9,'time at which suspended load transport ',
     &'recommences (sec)',/,
     &'PERBED',T9,'percentage of time spent in only bedload ',
     &'transport phase',/,
     &'PERSUSP',T9,'percentage of time spent in suspended load ',
     &'transport phase',/,
     &'C0',T9,'reference concentration at z0 (kg/m^3)',/,
     &'TAOCWS',T9,'averaged effective shear stress (Pa)',/,
     &'RHOW',T9,'water density (kg/m3)',/,
     &'VISC',T9,'dynamic viscosity of water (Pa s)',//)
132   FORMAT(
     &'In the output file *.pro (velocity and concentration profile',/,
     &'for non-cohesive sediments)',/,/,
     &'IRUN',T9,'identifier for the run (integer value)',/,
     &'Zi',T9,'height above the bed (m)',/,
     &'Ci',T9,'suspended sediment concentration at height Zi (kg/m**3)'/
     &'Ui',T9,'mean velocity at height Ui (m/s)',/,/,/,
     &'In the output file *.coh (results for cohesive sediments)',/,/,
     &'IRUN',T9,'identifier for the run (integer value)',/,
     &'ERR',T9,'error/warning code (see below)',/,
     &'TAU0EFF',T9,'effective To, includes drag reduction and TAOS ',
     &'(Pa)',/,
     &'TAOS',T9,'solid transmitted stress due to Ulva (Pa)',/,
     &'UNUSED',T9,'unused position',
     &'ZS',T9,'height change in bed surface (m)',/,
     &T9,'(positive: erosion, negative: deposition)',/,
     &'EDRATE',T9,'mean erosion/deposition rate (kg/m**2/s)',/,
     &T9,'(positive: erosion, negative: deposition)',/,
     &'TCONC',T9,'final total suspended sediment concentration ',
     &'(kg/m**3)',/,
     &'NBCONC',T9,'number of classes of suspended sediment',/,
     &'Ci',T9,'suspended sediment concentration for each class (kg/m3)'/
     &'bed characteristics (BEDCHA):',/,
     &'MAXBED',T9,'number of elements in Z, Tero and rho',/,
     &'Zi',T9,'depth below surface (m)',/,
     &'Tero_i',T9,'critical erosion stress at depth Zi (Pa)',/,
     &'rho_i',T9,'dry bulk density at depth Zi (kg/m**3)')
133   FORMAT(//,'Settling velocities (Ws) for each suspended sediment',/
     &,'class of cohesive sediment',/,
     &'Class  Ws (m/s)',/,21(I4,1X,F10.6,/))
134   FORMAT(/,'Error and warning codes',/
     &'Only one error/warning code is specified in the file *.dat',/,
     &'The full list of error/warning is given in the file *.txt',//,
     &'  0 No error or warning',//,
     &'Critical errors',/)
135   FORMAT(/,'Warnings',/)
136   FORMAT(I3,1X,A)
137   FORMAT(I3,1X,A,/,4X,A)

      END   ! end of subroutine EXAMPLE



C ======================================================================
C ======================================================================
C SUBROUTINE OPENFILE
C
C Open the input file (for batch mode) and the output files.
C If INFILE and OUTFILE are empty (not specified as command-line argument
C in batch mode, they are set to default names.
C Set OUTCOH to .FALSE. (output file for cohesive not already opened).
C For interactive (non-batch) mode, display the welcome message.
C Perform also all output operation that are done only once, i.e.,
C write the caution message to OUTDATA.txt and write the heading line
C to OUTDATA.dat
C
C The file units are
C     10 : input (batch mode)
C     11 : text output (file extension .txt)
C     12 : tabulated output (file extension .dat)
C     14 : sediment concentration profile for non-cohesive (file extension .pro)
C     15 : output for cohesive model (file extension .coh)

      SUBROUTINE OPENFILE(BATCH,FHEADER,ONEFILE,INFILE,OUTFILE,
     &                    CONSTFILE,OUTCOH,VERSION)
      IMPLICIT NONE
      LOGICAL BATCH            ! True for batch mode (read input from file)
      LOGICAL FHEADER          ! True if header line in output files
      LOGICAL ONEFILE          ! True if 1 instead of 3 tabular output files
      LOGICAL OUTCOH           ! True if file for cohesive output is already opened
      CHARACTER INFILE*(*)     ! Name of input file (batch mode)
      CHARACTER OUTFILE*(*)    ! Name of output file (batch mode)
      CHARACTER CONSTFILE*(*)! Name of file to modify the default parameters
      CHARACTER VERSION*(*)    ! version of the program
      CHARACTER OUT1*204,OUT2*204,OUT3*204,OUT4*204
      CHARACTER CH*600
      INTEGER LASTCH           ! function
      INTEGER I

      OUTCOH=.FALSE.
      IF (INFILE.EQ.' ') INFILE='INDATA.DAT'
      IF (OUTFILE.EQ.' ') OUTFILE='OUTDATA'
      OUT1=OUTFILE(1:LASTCH(OUTFILE)) // '.txt'
      OUT2=OUTFILE(1:LASTCH(OUTFILE)) // '.dat'
      OUT3=OUTFILE(1:LASTCH(OUTFILE)) // '.da2'
      OUT4=OUTFILE(1:LASTCH(OUTFILE)) // '.pro'

      IF(BATCH)OPEN(UNIT=10,FILE=INFILE,ERR=10)
      OPEN(UNIT=11,FILE=OUT1,ERR=20)
      OPEN(UNIT=12,FILE=OUT2,ERR=30)
      IF (.NOT. ONEFILE) THEN
         OPEN(UNIT=13,FILE=OUT3,ERR=40)
         OPEN(UNIT=14,FILE=OUT4,ERR=50)
      ENDIF

      WRITE (11,701,ERR=60)VERSION
      WRITE (11,702,ERR=60)
      WRITE (11,*,ERR=60)' '
      WRITE (11,703,ERR=60)
      WRITE (11,705,ERR=60)

      IF(CONSTFILE.NE.' ')CALL SETCCONST(CONSTFILE,11,' ') ! Modify default parameters

      IF (FHEADER) THEN
         IF (ONEFILE) THEN
            WRITE (CH,110)(I,I=1,12),(I,I=1,12),(I,I=1,12)
            CALL LINETAB(CH)
            WRITE (12,'(A)',ERR=70)CH(1:LASTCH(CH))
         ELSE
            WRITE(CH,120)
            CALL LINETAB(CH)
            WRITE (12,'(A)',ERR=70)CH(1:LASTCH(CH))

            WRITE(CH,130)
            CALL LINETAB(CH)
            WRITE (13,'(A)',ERR=80)CH(1:LASTCH(CH))

            WRITE (CH,140)(I,I=1,12),(I,I=1,12),(I,I=1,12)
            CALL LINETAB(CH)
            WRITE (14,'(A)',ERR=90)CH(1:LASTCH(CH))
         ENDIF
      ENDIF
110   FORMAT('IRUN ERR UB FCW UA U100 USTCWS USTCW Z0 ',
     &'FALL RHEIGHT RLENGTH C0A QS QSDIR SEDM SED SEDDIR ',
     &'AB WL USTCRB USTCRS USTUP Z0C PHIB PHI100 DELTACW USTCS',
     &' USTWS USTCWSE USTCWSB USTC USTW RPLCOEF TB1 TB2 TS1 TS2',
     &' PERBED PERSUSP C0 TAOCWS RHOW VISC',
     &' NB_Z',9(' z',I1),3(' z',I2),
     &   9(' C',I1),3(' C',I2),9(' U',I1),3(' U',I2))
120   FORMAT('IRUN ERR UB FCW UA U100 USTCWS USTCW Z0 ',
     &'FALL RHEIGHT RLENGTH C0A QS QSDIR SEDM SED SEDDIR')
130   FORMAT('IRUN ',
     &'AB WL USTCRB USTCRS USTUP Z0C PHIB PHI100 DELTACW USTCS',
     &' USTWS USTCWSE USTCWSB USTC USTW RPLCOEF TB1 TB2 TS1 TS2',
     &' PERBED PERSUSP C0 TAOCWS RHOW VISC')
140   FORMAT('IRUN NB_Z',9(' z',I1),3(' z',I2),
     &   9(' C',I1),3(' C',I2),9(' U',I1),3(' U',I2))

      IF (.NOT.BATCH) THEN
         WRITE(*,'(//)')
         WRITE(*,701)VERSION
         WRITE(*,703)
         WRITE(*,704)
         WRITE(*,705)
         WRITE(*,100)
         READ(*,'(A)')CH
         IF((ICHAR(CH).EQ.27).OR.(CH.EQ."Q").OR.(CH.EQ."q"))STOP
100      FORMAT('For instructions how to use the batch mode, start the '
     &   ,/,'program with the option "-?"',/,
     &   'For licence information, start the program with the ',
     %   'option "-L"',//,
     &   'Type <RETURN> to start')
C The same formats 701 and 702 exist in COMARG and in OPENFILE, they
C must be updated together !
701   FORMAT(
     &/,'SEDTRANS05 :  A SEDIMENT TRANSPORT MODEL (',A,')',/
     &/,'Copyright (C) 2005 Urs Neumeier, Christian Ferrarin, Carl Amos'
     &/,'and Georg Umgiesser.',
     &/,'Sedtrans05 was developed at the ISMAR-CNR in Venice and the',
     &/,'University of Southampton.',
     &/,'Previous versions of Sedtrans (1992 and 1996) were developed',
     &' at',
     &/,'the Geological Survey of Canada, Bedford Institute of ',
     &'Oceanography.',/)
702   FORMAT(
     &'This program is free software; you can redistribute it and/or',
     &' modify',
     &/,'it under the terms of the GNU General Public License as ',
     &'published by',
     &/,'the Free Software Foundation; either version 3 of the ',
     &'License, or',
     &/,'(at your option) any later version. See file "licence.txt" ',
     &'in the'
     &/,'Sedtrans05 directory for the full licence text, which can ',
     &'also be',
     &/,'found at http://www.gnu.org/licenses/',
     &/,
     &/,'This program is distributed in the hope that it will be ',
     &'useful,',
     &/,'but WITHOUT ANY WARRANTY; without even the implied warranty of'
     &/,'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the',
     &/,'GNU General Public License for more details.')

703   FORMAT('Description in Neumeier et al. (in press) ',
     &'Computers & Geosciences.',/,
     &'See also Li & Amos (2001) Computers & Geosciences, 27, 619-645.',
     &//,'The user should be familiar with the equations used ',
     &'and their limitations.')
704   FORMAT('This program is distributed in the hope that it will be ',
     &'useful,',/
     &,'but WITHOUT ANY WARRANTY.')
705   FORMAT('All dimensional variables are in SI units.',/)
      ENDIF

      RETURN

10    WRITE(*,*) 'Error during opening/reading file ',INFILE
      STOP
20    WRITE(*,*) 'Error during opening file ',OUT1
      STOP
30    WRITE(*,*) 'Error during opening file ',OUT2
      STOP
40    WRITE(*,*) 'Error during opening file ',OUT3
      STOP
50    WRITE(*,*) 'Error during opening file ',OUT4
      STOP
60    WRITE(*,*) 'Error during writing to file ',OUT1
      STOP
70    WRITE(*,*) 'Error during writing to file ',OUT2
      STOP
80    WRITE(*,*) 'Error during writing to file ',OUT3
      STOP
90    WRITE(*,*) 'Error during writing to file ',OUT4
      STOP

      END   !  end of subroutine OPENFILE
      
      
      
C ======================================================================
C ======================================================================
C SUBROUTINE READIN
C
C Read the input arguments for one call to SEDTRANS05
C Return ABORT: 0 sucessful input, 1 input aborted, 2 end of file for batch mode
C Check the input argument (returning error/warning code ERR)

      SUBROUTINE READIN(BATCH,INSALTEMP,ABORT,ERR,
     &IRUN,IOPT1,D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,RLINP,
     &BETA,RHOS,SALIN,TEMP,BEDCHA,MAXBED,CONC,TIMEDR,AULVA)

      IMPLICIT NONE
      INCLUDE "sed_param.h" ! declaration of the COMMON block /WSCLASS/
      LOGICAL BATCH
      LOGICAL INSALTEMP  ! True if input is Temp./Salinity, otherwise water density/dynam.viscosity
      INTEGER ABORT
      INTEGER ERR(20)           ! Error/warning code from CHECKIN
      INTEGER IOPT1,IRUN
      DOUBLE PRECISION D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,RLINP,
     &BETA,RHOS,SALIN,TEMP,TIMEDR,AULVA
      INTEGER MAXBED          ! upper bound of first dimension of BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
      DOUBLE PRECISION CONC(NBCONC) !array of suspended sediment concentration (mg/l)

      CHARACTER*30 TEXT1
      LOGICAL FIRST
      INTEGER I, J, NBED
      DOUBLE PRECISION RIRUN,RIOPT1,RNBED   ! temporary real variables for
                                            ! IRUN, IOPT1 and NBED

      FIRST=(TEMP.EQ.-999.)
      IF(FIRST)THEN
         RHOS=2650d0
         SALIN=35d0
      ENDIF
      ABORT=0
      ERR(1)=0

      IF (BATCH) THEN
C Batch mode, read data from the input file
630      TEMP=-99d0
         DO 635,I=1,MAXBED
635      BEDCHA(I,3)=-99d0
         READ(10,*,ERR=630,END=620)RIRUN,RIOPT1,D,UZ,Z,CDIR,HT,PER,
     &    WDIR,GD,RHINP,RLINP,BETA,RHOS,SALIN,TEMP,
     &      ( TIMEDR,AULVA,CONC,RNBED,
     &         (BEDCHA(I,1),I=1,MIN(NINT(RNBED),MAXBED)),
     &         (BEDCHA(I,2),I=1,MIN(NINT(RNBED),MAXBED)),
     &         (BEDCHA(I,3),I=1,MIN(NINT(RNBED),MAXBED)),
     &                                    J=1,NINT(RIOPT1)/7 )
         IRUN=NINT(RIRUN)
         IOPT1=NINT(RIOPT1)
         NBED=NINT(RNBED)
         IF(TEMP.EQ.-99d0) THEN      ! some input values are missing
            WRITE(11,310)IRUN
            GOTO 630
         ELSEIF(IOPT1.EQ.7)THEN
            IF(NBED.LT.1 .OR. NBED.GT.MAXBED) THEN
               ERR(1)=15
               ERR(2)=0
               ABORT=1   ! abort input for this run
               RETURN
            ELSE
               IF(BEDCHA(NBED,3).EQ.-99d0) THEN
                  WRITE(11,310)IRUN
                  GOTO 630
               ENDIF
            ENDIF
         ENDIF
310   FORMAT(/,60('*')/,' Some input values are missing for run ',I5,
     &'.',/,'This run is not processed.',60('*'))

         IF (IOPT1.EQ.7) THEN
            DO 640,I=NBED+1,MAXBED
               BEDCHA(I,1)=0
640         CONTINUE
         ENDIF

      ELSE

C INTERACTIVE DATA ENTRY
         IRUN=IRUN+1
         WRITE (*,10)IRUN
10       FORMAT(75('*'),//,'This is run number :',I4)

         WRITE (*,20)
20       FORMAT('If you wish to abort a run, enter -99 as response',/,
     @   'to any of the following questions.')

         IF (.NOT.FIRST) WRITE(*,21)
21       FORMAT('Press ENTER for any question to keep the value of the',
     @   ' previous run.')

30       WRITE (*,31)
31       FORMAT (/,' Choose between:',/,
     @   '   1 - ENGELUND-HANSEN (1967) total load equation',/,
     @   '   2 - EINSTEIN-BROWN (1950) bedload equation',/,
     @   '   3 - BAGNOLD (1963) total load equation',/,
     @   '   4 - YALIN (1963) bedload equation',/,
     @   '   5 - VAN RIJN (1993) bedload equation',/,
     @   '   7 - Cohesive sediment algorithm',/,
     @   ' enter 1,2,3,4,5 OR 7: ')
         READ(*,32)TEXT1
32       FORMAT(A)
         IF(TEXT1.EQ.' ') THEN
            IF(FIRST) THEN
               GOTO 30
            ELSE
               WRITE(*,*)IOPT1
            ENDIF
         ELSE
            READ(TEXT1,*,ERR=30)IOPT1
         ENDIF
         IF (IOPT1 .EQ. -99) GO TO 610
         IF (IOPT1 .LT. 1 .OR. IOPT1 .GT. 7 .OR. IOPT1.EQ.6) GO TO 30

         IF (IOPT1.EQ.7) THEN
            WRITE(*,33)
            READ(*,32)TEXT1
            GOTO 610
         ENDIF
33    FORMAT(/,
     &'The cohesive sediment method can only be used in batch mode',/
     &'with data input from a file.',/,
     &'Start Sedtrans05 with the command-line option "-?" for ',/
     &'additional explanations.'//,'Type <RETURN> to continue')

40       CALL READONE(' Enter water depth (m): ',D,FIRST)
         IF ( D .EQ. -99.) GO TO 610
         IF (D.LE.0.) THEN
            WRITE(*,650)
650         FORMAT(47('*'),/,'Incorrect value !   Try again')
            GOTO 40
         ENDIF

50       CALL READONE(' Enter height of current measurement (m): ',
     &            Z,FIRST)
         IF (Z .EQ. -99.) GO TO 610
         IF (Z.LE.0. .OR. Z.GT.D) THEN
            WRITE(*,650)
            GOTO 50
         ENDIF

60       CALL READONE(' Enter current speed (m/s): ',UZ,FIRST)
         IF (UZ .EQ. -99.) GO TO 610
         IF (UZ.LT.0.) THEN
            WRITE(*,650)
            GOTO 60
         ENDIF

70       CALL READONE(' Enter current direction (degrees north): ',
     &         CDIR,FIRST)
         IF (CDIR .EQ. -99.) GO TO 610
         IF (CDIR.LT.0. .OR. CDIR.GT.360.) THEN
            WRITE(*,650)
            GOTO 70
         ENDIF

80       CALL READONE(' Enter wave height (m): ',HT,FIRST)
         IF (HT .EQ. -99.) GO TO 610
         IF (HT.GT.2d0*D) THEN
            WRITE(*,650)
            GOTO 80
         ENDIF

         IF (HT.GT.0) THEN
90          CALL READONE(' Enter wave period (seconds): ',PER,FIRST)
            IF (PER .EQ. -99.) GO TO 610
            IF (PER.LE.0.) THEN
               WRITE(*,650)
               GOTO 90
            ENDIF

100         CALL READONE(' Enter wave direction (degrees north): ',
     &            WDIR,FIRST)
            IF (WDIR .EQ. -99.) GO TO 610
            IF (WDIR.LT.0. .OR. WDIR.GT.360.) THEN
               WRITE(*,650)
               GOTO 100
            ENDIF
         ELSE
            PER=0d0
            WDIR=0d0
         ENDIF

110      CALL READONE(' Enter grain size (m)',GD,FIRST)
         IF (GD .EQ. -99.) GO TO 610
         IF (GD.LT.0. .OR. GD.GT.0.256) THEN
            WRITE(*,650)
            GOTO 110
         ENDIF

120      CALL READONE(' Enter ripple height (m)',RHINP,FIRST)
         IF (RHINP .EQ. -99.) GO TO 610
         IF (RHINP.LT.0. .OR. RHINP.GE.D) THEN
            WRITE(*,650)
            GOTO 120
         ENDIF

         IF(RHINP.GT.0) THEN
125         CALL READONE(' Enter ripple length (m)',RLINP,FIRST)
            IF (RLINP .EQ. -99.) GO TO 610
            IF (RLINP.LE.0.) THEN
               WRITE(*,650)
               GOTO 125
            ENDIF
         ELSE
            RLINP=0d0
         ENDIF

130      CALL READONE(' Enter bed slope (degree): ',BETA,FIRST)
         IF (BETA .EQ. -99.) GO TO 610
         IF (BETA.LT.-35. .OR. BETA.GT.35.) THEN
            WRITE(*,650)
            GOTO 130
         ENDIF

140      CALL READONE(' Enter grain density (default=2650, kg/m**3): ',
     &         RHOS,.FALSE.)
         IF (RHOS .EQ. -99.) GO TO 610
         IF (RHOS.LE.1025. .OR. RHOS.GE.22000.) THEN
            WRITE(*,650)
            GOTO 140
         ENDIF

         IF (INSALTEMP) THEN

150         CALL READONE(' Enter salinity (default=35): ',
     &            SALIN,.FALSE.)
            IF (SALIN .EQ. -99.) GO TO 610
            IF (SALIN.LT.0. .OR. SALIN.GT.42.) THEN
               WRITE(*,650)
               GOTO 150
            ENDIF

160         CALL READONE(' Enter temperature (degrees Celsius): ',
     &            TEMP,FIRST)
            IF (TEMP .EQ. -99.) GO TO 610
            IF (TEMP.LE.0. .OR. TEMP.GT.35.) THEN
               WRITE(*,650)
               GOTO 160
            ENDIF

          ELSE
          
155         CALL READONE(' water density (kg/m3): ',
     &            SALIN,FIRST)
            IF (SALIN .EQ. -99.) GO TO 610
            IF (SALIN.LT.900. .OR. SALIN.GT.1100.) THEN
               WRITE(*,650)
               GOTO 155
            ENDIF

165         CALL READONE(' Enter dynamic viscosity of water (Pa s): ',
     &            TEMP,FIRST)
            IF (TEMP .EQ. -99.) GO TO 610
            IF (TEMP.LE.0.0020 .OR. TEMP.GE.0.0005) THEN
               WRITE(*,650)
               GOTO 165
            ENDIF

         ENDIF

      ENDIF     ! end of interactive input

C Write the input values to the screen (not in batch mode) and the text-output file
      WRITE(11,301)IRUN,IOPT1,D,UZ,Z,CDIR,HT,PER,WDIR,GD,
     &      RHINP,RLINP,BETA,RHOS
      IF (INSALTEMP) THEN
         WRITE(11,302)SALIN,TEMP
      ELSE
         WRITE(11,303)SALIN,TEMP
      ENDIF
      IF(.NOT.BATCH) THEN
         WRITE(*,301)IRUN,IOPT1,D,UZ,Z,CDIR,HT,PER,WDIR,GD,
     &         RHINP,RLINP,BETA,RHOS,SALIN,TEMP
      ENDIF
      IF(IOPT1.EQ.7) THEN
         WRITE(11,304)TIMEDR,AULVA,
     &      (I,CONC(I),I+7,CONC(I+7),I+14,CONC(I+14),I=1,7)
         WRITE(11,305)
         WRITE(11,306)(BEDCHA(I,1),BEDCHA(I,2),BEDCHA(I,3),I=1,NBED)
      ENDIF
301   FORMAT(/,60('*')//,' The input values for run',I5,' are :',//,
     &T5,'IOPT1 sediment transport equation: ',10X,I1,/,
     &T5,'D     water depth (m):',16X,F8.2,/,
     &T5,'UZ    current speed (m/s):',14X,F6.2,/,
     &T5,'Z     height of current measurement (m):',F6.2,/,
     &T5,'CDIR  current direction (degrees north):',F6.1,/,
     &T5,'HT    wave height (m):',18X,F6.2,/,
     &T5,'PER   wave period (seconds):',12X,F6.2,/,
     &T5,'WDIR  wave direction (degrees north):',3X,F6.1,/,
     &T5,'GD    grain size (m):',16X,F9.6,/,
     &T5,'RHINP ripple height (m):',16X,F6.2,/,
     &T5,'RLINP ripple length (m):',16X,F6.2,/,
     &T5,'BETA  bed slope (degree):',15X,F6.1,/,
     &T5,'RHOS  grain density (kg/m**3):',8X,F8.1)
302   FORMAT(
     &T5,'SALIN salinity:',23X,F8.1,/,
     &T5,'TEMP  temperature (degrees Celsius):',4X,F6.1)
303   FORMAT(
     &T5,'RHOW water density (kg/m3):',11X,F8.1,/,
     &T5,'VISC  dynamic viscosity of water (Pa s):',F6.4)
304   FORMAT(
     &T5,'TIMEDR,deposition/erosion duration (s):',1X,F6.1,/,
     &T5,'AULVA bed area covered by Ulva:',10X,F5.3,/,
     &T5,'CONC  SSC for each Ws class (kg/m**3)',
     &7(/,T7,3(I2,' :',F8.5,4X)))
305   FORMAT(T5,'BEDCHA     z         Terosion   density')
306   FORMAT(T12,F10.4,F10.4,F10.1)

CCCC      IF(GD.EQ.0. .AND. IOPT1.EQ.7) GD=1d-5

      CALL CHECKIN(INSALTEMP,ERR,D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,
     &RLINP,BETA,RHOS,SALIN,TEMP,IOPT1,TIMEDR,AULVA,
     &BEDCHA,MAXBED,CONC)

      IF(ERR(1).NE.0 .AND. ERR(1).LT.100) ABORT=1   ! abort input for this run

      RETURN

610   ABORT=1     ! abort input for this run
      RETURN
620   ABORT=2     ! end of file for batch mode
      RETURN
      END    ! end of subroutine READIN


C ======================================================================
C ======================================================================
C SUBROUTINE READONE
C
C Display the message TXT and read the double precision variable PARAM
C If an error occurs during interpretation of the input, start again
C If FIRST is true, then an empty input (simple return) does not modify
C the previous value of PARAM
      SUBROUTINE READONE(TEXT,PARAM,FIRST)
      IMPLICIT NONE
      CHARACTER TEXT*(*),CH*30
      DOUBLE PRECISION PARAM
      LOGICAL FIRST

10    WRITE(*,100)TEXT
      READ(*,101)CH
100   FORMAT(/,A)
101   FORMAT(A)
      IF(CH.EQ.' ') THEN
         IF(FIRST) THEN
            GOTO 10
         ELSE
            WRITE(*,*)PARAM
         ENDIF
      ELSE
         READ(CH,*,ERR=20)PARAM
      ENDIF
      RETURN

20    WRITE(*,650)
650   FORMAT(47('*'),/,'Incorrect value !   Try again')
      GOTO 10
      END   ! end of subroutine READONE



C ======================================================================
C ======================================================================
C  SUBROUTINE CHECKIN
C
C Checks the validity of the input arguments and reports an error/warning
C code ERR and a message ERRTXT for the first error/warning observed.
C First checks the range of the input arguments, then whether the grain-size
C is adapted for the selected equation. Returns after the first
C observed error/warning, this mean several errors/warning may exist,
C but only the first is reported.
C ERR = 0    --> no error/warning
C ERR = 1-99 --> error, SEDTRANS05 should not be called
C ERR > 100  --> warning, SEDTRANS05 can be called, but warning should be reported

C List of possible errors/warnings
C 1   Incorrect input argument D
C 2   Incorrect input argument UZ
C 3   Incorrect input argument Z
C 4   Incorrect input argument CDIR
C 5   Incorrect input argument HT
C 6   Incorrect input argument PER
C 7   Incorrect input argument WDIR
C 8   Incorrect input argument GD
C 9   Incorrect input argument RHINP
C 10  Incorrect input argument RLINP
C 11  Incorrect input argument BETA
C 12  Incorrect input argument SALIN
C 13  Incorrect input argument RHOS
C 14  Incorrect input argument AULVA
C 15  Incorrect value for NBED
C 16  Incorrect input argument TIMEDR
C 17  Incorrect input argument BEDCHA
C 18  Incorrect input argument CONC
C 19  Incorrect input argument TEMP
C 30  Incorrect input argument IOPT1
C 101 ENGELUND-HANSEN formula not recommended for use with sediments finer than 0.15 mm
C 102 EINSTEIN-BROWN formula is based on laboratory experiments using sediments with grain sizes of 0.3 to 28.6 mm
C 103 BAGNOLD formula is based on laboratory tests with grain sizes between 0.18 and 0.45 mm
C 104 YALIN formula is not recommended for use with sediments smaller than 0.2mm, based on the results of sensitivity analyses
C 105 ACKERS-WHITE formula is based on laboratory experiments using sediments with grain sizes of 0.05 to 29.1 mm
C 107 The cohesive algorithm is not intended for non-cohesive sediments

C IOPT1 = SEDIMENT TRANSPORT PREDICTOR OPTION NUMBER
C            1 - ENGELUND-HANSEN (1967) TOTAL LOAD EQUATION
C            2 - EINSTEIN-BROWN (1950) BEDLOAD EQUATION
C            3 - BAGNOLD (1963) TOTAL LOAD EQUATION
C            4 - YALIN (1963) BEDLOAD EQUATION
C            5 - VAN RIJN (1993) BEDLOAD EQUATION
C            7 - Cohesive sediments algorithm

      SUBROUTINE CHECKIN(INSALTEMP,ERR,D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,
     &RLINP,BETA,RHOS,SALIN,TEMP,IOPT1,TIMEDR,AULVA,
     &BEDCHA,MAXBED,CONC)
      IMPLICIT NONE
C **************** input variables **********************
      LOGICAL INSALTEMP  ! True if input is Temp./Salinity, otherwise water density/dynam.viscosity
      DOUBLE PRECISION D,UZ,Z,CDIR,HT,PER,WDIR,GD,RHINP,RLINP,BETA,
     @RHOS,SALIN,TEMP,TIMEDR,AULVA
      INTEGER IOPT1
      INTEGER MAXBED          ! upper bound of first dimension of BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
      INTEGER NBCONC
      PARAMETER (NBCONC = 21) ! number of element in CONC
      DOUBLE PRECISION CONC(NBCONC) !array of suspended sediment concentration (mg/l)
C **************** output variables **********************
      INTEGER ERR(20)          ! Error/Warning code
C **************** intermediate variables **********************
      DOUBLE PRECISION TCONC
      LOGICAL CHECKBEDCHA   ! function
      INTEGER I,NB

      DO 10,NB=1,20
10    ERR(NB)=0
      NB=1

C Errors for input argument outside of accetable range
      IF (D.LE.0.) THEN
         ERR(NB)=1
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (UZ.LT.0.) THEN
         ERR(NB)=2
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (Z.LE.0. .OR. Z.GT.D) THEN
         ERR(NB)=3
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (CDIR.LT.0. .OR. CDIR.GT.360.) THEN
         ERR(NB)=4
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (HT.LT.0 .OR. HT.GE.(2d0*D)) THEN
         ERR(NB)=5
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (HT.GT.0) THEN
         IF (PER.LT.0.) THEN
            ERR(NB)=6
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
         IF (WDIR.LT.0. .OR. WDIR.GT.360.) THEN
            ERR(NB)=7
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
      ENDIF
      IF (GD.LT.0. .OR. GD.GT.0.256) THEN
         ERR(NB)=8
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (RHINP.GE.D .OR. RHINP.LT.0.) THEN
         ERR(NB)=9
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (RLINP.LT.0. .OR. (RLINP.EQ.0. .AND. RHINP.GT.0.)) THEN
         ERR(NB)=10
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (BETA.LT.-35. .OR. BETA.GT.35.) THEN
         ERR(NB)=11
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF ((SALIN.LT.0. .OR. SALIN.GT.42.) .AND. INSALTEMP) THEN
         ERR(NB)=12
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (RHOS.LE.1025. .OR. RHOS.GE.22000.) THEN
         ERR(NB)=13
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      

      
      IF (IOPT1.EQ.7) THEN
         IF (AULVA.LT.0. .OR. AULVA.GT.1.) THEN
            ERR(NB)=14
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
         IF (TIMEDR.LE.0.) THEN
            ERR(NB)=16
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
         IF (.NOT.CHECKBEDCHA(BEDCHA,MAXBED)) THEN
            ERR(NB)=17
            NB=NB+1
         IF(NB.GT.20) RETURN
         ENDIF
         TCONC=0.0d0
         DO 20, I=1,NBCONC
            IF (CONC(I).LT.0) GOTO 30
            TCONC=TCONC+CONC(I)
20       CONTINUE
         IF (TCONC.LE.1500) GOTO 40
30          ERR(NB)=18
            NB=NB+1
         IF(NB.GT.20) RETURN
40       CONTINUE
      ENDIF
      IF ((TEMP.LE.0. .OR. TEMP.GT.35.) .AND. INSALTEMP) THEN
         ERR(NB)=19
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      IF (IOPT1.LT.1 .OR. IOPT1.GT.7 .OR. IOPT1.EQ.6) THEN
         ERR(NB)=30
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF

C Warnings for grain-size not adapted for selected formula
      GOTO(210,220,230,240,250,500,270)IOPT1
 210  IF (GD.LT.0.00015) THEN
         ERR(NB)=101
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 220  IF (GD .LT. 0.0003 .OR. GD .GT. .0286) THEN
         ERR(NB)=102
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 230  IF (GD.LT.0.00018 .OR. GD.GT.0.00045)THEN
         ERR(NB)=103
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 240  IF (GD.LT.0.0002)THEN
         ERR(NB)=104
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 250  IF (GD.LT.0.0002 .OR. GD.GT.0.002)THEN
         ERR(NB)=105
         NB=NB+1
         IF(NB.GT.20) RETURN
      ENDIF
      GOTO 500
 270  IF (GD.GT.0.00006)THEN
C         ERR(NB)=107
C         NB=NB+1
C         IF(NB.GT.20) RETURN
      ENDIF

 500  CONTINUE

      END    ! end of subroutine CHECKIN



C ======================================================================
C ======================================================================
C  SUBROUTINE ERRTEXT
C
C Return the text corresponding to the error code, which was produce by CHECKIN
C
      SUBROUTINE ERRTEXT(ERR,ERRTXT)
      INTEGER ERR          ! Error/Warning code
      CHARACTER*140 ERRTXT ! Error/Warning text corresponding to ERR, if the
                           ! text is longer than 75 characters, ERRTXT(1:75) is
                           ! the 1st line and ERRTXT(76:140) is the second line
      CHARACTER STXT*25
      PARAMETER (STXT='Incorrect input argument ')

      ERRTXT=' '
      IF(ERR.GE.1 .AND. ERR.LE.24) THEN
         GOTO (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,
     &            20,21,22,23,24),ERR
1        ERRTXT=STXT // 'D'
         GOTO 500
2        ERRTXT=STXT // 'UZ'
         GOTO 500
3        ERRTXT=STXT // 'Z'
         GOTO 500
4        ERRTXT=STXT // 'CDIR'
         GOTO 500
5        ERRTXT=STXT // 'HT'
         GOTO 500
6        ERRTXT=STXT // 'PER'
         GOTO 500
7        ERRTXT=STXT // 'WDIR'
         GOTO 500
8        ERRTXT=STXT // 'GD'
         GOTO 500
9        ERRTXT=STXT // 'RHINP'
         GOTO 500
10       ERRTXT=STXT // 'RLINP'
         GOTO 500
11       ERRTXT=STXT // 'BETA'
         GOTO 500
12       ERRTXT=STXT // 'SALIN'
         GOTO 500
13       ERRTXT=STXT // 'RHOS'
         GOTO 500
14       ERRTXT=STXT // 'AULVA'
         GOTO 500
15       ERRTXT='Incorrect value for NBED'
         GOTO 500
16       ERRTXT=STXT // 'TIMEDR'
         GOTO 500
17       ERRTXT=STXT // 'BEDCHA'
         GOTO 500
18       ERRTXT=STXT // 'CONC'
         GOTO 500
19       ERRTXT=STXT // 'TEMP'
         GOTO 500
20       ERRTXT=STXT // 'DT'
         GOTO 500
21       ERRTXT=STXT // 'USTCWS'
         GOTO 500
22       ERRTXT=STXT // 'ARG4'
         GOTO 500
23       ERRTXT=STXT // 'OBS2'
         GOTO 500
24       ERRTXT=STXT // 'ROT'
         GOTO 500

      ELSEIF(ERR.EQ.30) THEN
         ERRTXT=STXT // 'IOPT1'
         GOTO 500
      ELSEIF(ERR.GE.101 .AND. ERR.LE.107) THEN
         GOTO(101,102,103,104,105,500,107), (ERR-100)
101      ERRTXT(1:75)='ENGELUND-HANSEN formula not recommended for '//
     &      'use with sediments finer than'
         ERRTXT(76:140)='0.15 mm'
         GOTO 500
102      ERRTXT(1:75)='EINSTEIN-BROWN formula is based on laboratory '//
     &      'experiments using sediments'
         ERRTXT(76:140)='with grain sizes of 0.3 to 28.6 mm'
         GOTO 500
103      ERRTXT(1:75)='BAGNOLD formula is based on laboratory tests '//
     &      'with grain sizes between 0.18'
         ERRTXT(76:140)='and 0.45 mm'
         GOTO 500
104      ERRTXT(1:75)='YALIN formula is not recommended for use with '//
     &      'sediments smaller than 0.2mm,'
         ERRTXT(76:140)='based on the results of sensitivity analyses'
         GOTO 500
105      ERRTXT(1:75)='VAN RIJN formula is based on laboratory '//
     &      'experiments using sediments'
         ERRTXT(76:140)='with grain sizes of 0.2 to 2.0 mm'
         GOTO 500
107      ERRTXT='The cohesive algorithm is not intended for '//
     &      'non-cohesive sediments'
         GOTO 500
      ENDIF
500   CONTINUE
      END   ! end of subroutine ERRTEXT



C ======================================================================
C ======================================================================
C SUBROUTINE WRITEOUT
C
C Write the result of SEDTRANS05 and BEDFORM to the output files (and to the
C screen in non-batch mode).
C The first time that there is an output of for cohesive sediment,
C open also the cohesive output file,

      SUBROUTINE WRITEOUT(BATCH,FHEADER,ONEFILE,OUTCOH,OUTFILE,ERR,
     &BEDTXT,IRUN,IOPT1,CONC,BEDCHA,MAXBED,RHOW,
     &UB,FCW,UA,U100,USTCWS,USTCW,Z0,RHEIGHT,RLENGTH,QS,QSDIR,SEDM,
     @SED,SEDDIR,FALL,C0A,EDRATE,ZS,TAOS,TCONC,VISC,
     @USTCRB,USTCRS,USTUP)
      IMPLICIT NONE
      INCLUDE "sed_param.h" ! declaration of the COMMON blocks

C input variables
      LOGICAL BATCH            ! True for batch mode (read input from file)
      LOGICAL FHEADER          ! True if header line in output files
      LOGICAL ONEFILE          ! True if 1 instead of 3 tabular output files
      LOGICAL OUTCOH           ! True if file for cohesive output is already opened
      CHARACTER OUTFILE*80     ! Name of output file (batch mode)
      INTEGER ERR(20)          ! Error/Warning code (from CHECKIN)
      INTEGER IRUN             ! ID for each run
      CHARACTER*80 BEDTXT(7)   ! Text of bedform prediction

      INTEGER IOPT1
      INTEGER MAXBED          ! upper bound of first dimension of BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
      DOUBLE PRECISION CONC(NBCONC) !array of suspended sediment concentration (mg/l)
      DOUBLE PRECISION RHOW,
     &UB,FCW,UA,U100,USTCWS,USTCW,Z0,RHEIGHT,RLENGTH,QS,QSDIR,SEDM,
     @SED,SEDDIR,FALL,C0A,EDRATE,ZS,TAOS,TCONC,VISC,
     @USTCRB,USTCRS,USTUP
C intermediate variable
      INTEGER LASTCH          ! function
      CHARACTER OUT5*84       ! file name for the cohesive output
      CHARACTER CH*2000,CH2*1000
      CHARACTER FORMAT1*170,FORMAT2*80,EQUATION*50,TEXT1*30
      CHARACTER*140 ERRTXT ! Error/Warning text corresponding to ERR, if the
                           ! text is longer than 75 characters, ERRTXT(1:75) is
                           ! the 1st line and ERRTXT(76:140) is the second line
      INTEGER I,J
      INTEGER NBPROF          ! number of point in the SSC and U profile
      PARAMETER (NBPROF=12)
      DOUBLE PRECISION Z2(NBPROF) ! copy of Z

C Open cohesive output-file if cohesive model and output-file not already open
      IF(IOPT1.EQ.7 .AND. (.NOT.OUTCOH) ) THEN
         OUT5=OUTFILE(1:LASTCH(OUTFILE)) // '.coh'
         OPEN(UNIT=15,FILE=OUT5,ERR=300)
         OUTCOH=.TRUE.
         
         IF (FHEADER) THEN
            IF(NBCONC.LT.10) THEN
               WRITE(FORMAT1,100)NBCONC
            ELSE
               WRITE(FORMAT1,101)NBCONC-9
            ENDIF
            IF(MAXBED.LT.10) THEN
               WRITE(FORMAT2,102)MAXBED,MAXBED,MAXBED
            ELSE
               WRITE(FORMAT2,103)MAXBED-9,MAXBED-9,MAXBED-9
            ENDIF
            FORMAT1=FORMAT1(1:LASTCH(FORMAT1)) // FORMAT2
            WRITE (CH,FORMAT1)(I,I=1,NBCONC),(I,I=1,MAXBED),
     &                  (I,I=1,MAXBED),(I,I=1,MAXBED)
            CALL LINETAB(CH)
            WRITE (15,'(A)',ERR=330)CH(1:LASTCH(CH))
         ENDIF
      ENDIF
100   FORMAT('(''IRUN ERR TAU0EFF TAOS UNUSED ZS EDRATE TCONC ',
     &   'NBCONC'',',I1,'('' c'',I1),'' MAXBED'',')
101   FORMAT('(''IRUN ERR TAU0EFF TAOS UNUSED ZS EDRATE TCONC ',
     &   'NBCONC'',9('' c'',I1),',I2,'('' c'',I2),'' MAXBED'',')
102   FORMAT(I1,'('' z'',I1),',I1,'('' Tero'',I1),',I1,'('' rho'',I1))')
103   FORMAT('9('' z'',I1),',I2,'('' z'',I2),',
     &   '9('' Tero'',I1),',I2,'('' Tero'',I2),',
     &   '9('' rho'',I1),',I2,'('' rho'',I2))')

C Write the text ouput to the *.txt file
      WRITE(11,110,ERR=310)IRUN
      IF (.NOT.BATCH) WRITE(*,110)IRUN

      DO 10,I=1,20    ! write errors/warnings
         IF (ERR(I).EQ.0) GOTO 15
         CALL ERRTEXT(ERR(I),ERRTXT)
         IF (ERR(I).LT.100) THEN
            TEXT1='*********** ERROR ************'
         ELSE
            TEXT1='********** WARNING ***********'
         ENDIF
         IF (.NOT.BATCH) THEN
            IF (LASTCH(ERRTXT).LE.75) THEN
               WRITE(*,601)TEXT1,ERRTXT(1:LASTCH(ERRTXT))
            ELSE
               WRITE(*,602)TEXT1,ERRTXT(1:75),ERRTXT(76:LASTCH(ERRTXT))
            ENDIF
         ENDIF
         IF (LASTCH(ERRTXT).LE.75) THEN
            WRITE(11,601)TEXT1,ERRTXT(1:LASTCH(ERRTXT))
         ELSE
            WRITE(11,602)TEXT1,ERRTXT(1:75),ERRTXT(76:LASTCH(ERRTXT))
         ENDIF
10    CONTINUE
601   FORMAT(//,A,/,A,/)
602   FORMAT(//,A,/,A,/,A,/)
15    IF (ERR(1).NE.0 .AND. ERR(1).LT.100) THEN
         IF(.NOT.BATCH)RETURN  ! do not write output files if interactive mode
         IF(ONEFILE) THEN
            WRITE(CH,*)IRUN,ERR(1),(0,I=1,(43+NBPROF*3) )
            CALL LINETAB(CH)
            WRITE(12,'(A)',ERR=320)CH(1:LASTCH(CH)) ! *.dat file
         ELSE
            WRITE(CH,*)IRUN,ERR(1),(0,I=1,16)
            CALL LINETAB(CH)
            WRITE(12,'(A)',ERR=320)CH(1:LASTCH(CH)) ! *.dat file
            WRITE(CH,*)IRUN,(0,I=1,26)
            CALL LINETAB(CH)
            WRITE(13,'(A)',ERR=330)CH(1:LASTCH(CH)) ! *.da2 file
            WRITE(CH,*)IRUN,NBPROF,(0,I=1,NBPROF*3)
            CALL LINETAB(CH)
            WRITE(14,'(A)',ERR=340)CH(1:LASTCH(CH)) ! *.pro file
         ENDIF
         IF(IOPT1.NE.7) RETURN
         WRITE(CH,*)IRUN,(0,I=1,(7+NBCONC+3*MAXBED))
         CALL LINETAB(CH)
         WRITE(15,'(A)',ERR=350)CH(1:LASTCH(CH)) ! *.coh file
         RETURN
      ENDIF

      WRITE(11,111)UB,AB,WL,U100,UA,PHIB
      IF(IOPT1.NE.7) THEN
         WRITE(11,112)PERBED,PERSUSP,(100d0-PERBED-PERSUSP),
     &               SEDDIR,SED,SEDM
      ELSE
         WRITE(11,113)SEDDIR,SEDM
      ENDIF
      IF(IOPT1.EQ.1) THEN
         EQUATION='ENGELUND-HANSEN (1967) total load equation'
      ELSEIF(IOPT1.EQ.2) THEN
         EQUATION='EINSTEIN-BROWN (1950) bedload equation'
      ELSEIF(IOPT1.EQ.3) THEN
         EQUATION='BAGNOLD (1963) total load equation'
      ELSEIF(IOPT1.EQ.4) THEN
         EQUATION='YALIN (1963) bedload equation'
      ELSEIF(IOPT1.EQ.5) THEN
         EQUATION='VAN RIJN (1993) bedload equation'
      ELSEIF(IOPT1.EQ.7) THEN
         EQUATION='Cohesive sediment algorithm'
      ENDIF
      WRITE(11,114,ERR=310)EQUATION(1:LASTCH(EQUATION))

      IF (.NOT.BATCH) THEN
         WRITE(*,111)UB,AB,WL,U100,UA,PHIB
         IF(IOPT1.NE.7) THEN
            WRITE(*,112)PERBED,PERSUSP,(100d0-PERBED-PERSUSP),
     &                  SEDDIR,SED,SEDM
         ELSE
            WRITE(*,113)SEDDIR,SEDM
         ENDIF
         WRITE(*,114)EQUATION(1:LASTCH(EQUATION))
      ENDIF
      DO 20, I=1,7
         IF(BEDTXT(I).EQ.' ') GOTO 30
         WRITE(11,'(A)')BEDTXT(I)(1:LASTCH(BEDTXT(I)))
         IF (.NOT.BATCH) WRITE(*,'(A)')BEDTXT(I)(1:LASTCH(BEDTXT(I)))
20    CONTINUE
110   FORMAT(/,' The results for run',I5,' are :',/)
111   FORMAT(
     &' Max. wave-induced bottom horizontal particle velocity     =',
     &F7.3,' m/s',/,
     &' Max. wave-induced bottom horizontal particle displacement =',
     &F7.3,' m',/,
     &' Wavelength  (all 3 parameters from linear wave theory)    =',
     &F6.2,' m',//,
     &' Curent speed at 1 m above seabed              =',F7.2,' m/s',/,
     &' Current at the top of the wave boundary layer =',F7.2,' m/s',/,
     &' Angle between wave and current direction',/,
     &' within wave boundary layer',20X,'=',F7.2,' degrees',/)
112   FORMAT(' Percent of time in only bedload transport phase',
     &T59,'=',F7.2,' %',/,
     &' Percent of time in only suspended load transport phase',T59,'=',
     &F7.2,' %',/,
     &' Percent of time with no transport',T59,'=',F7.2,' %',//,
     &' Direction of net sediment transport           =',
     &F7.2,' degrees north',/,
     &' Time-averaged sediment transport              = ',
     &G9.4,' m^3/s/m',/,
     &49X,G9.4,' kg/s/m')
113   FORMAT(' Direction of net sediment transport           =',
     &F7.2,' degrees north',/,
     &' Time-averaged sediment transport              = ',
     &G9.4,' kg/s/m')
114   FORMAT('    (',A,')',//,
     &' EXPECTED BEDFORMS ARE (Amos, 1990; Li & Amos (1998):')

30    IF (ONEFILE) THEN

C write ALL the tabulated output to the *.dat file
         IF(IOPT1.NE.7) THEN
            DO 40, I=1,NBPROF
               Z2(I)=ZZ(I);
40          CONTINUE
            WRITE(CH2,*)NBPROF,Z2,CZ,UUZ
         ELSE                     ! No profile for cohesive sediment
            WRITE(CH2,*)NBPROF,(0,I=1,NBPROF*3)
         ENDIF
         WRITE(CH,*)IRUN,ERR(1),UB,FCW,UA,U100,USTCWS,USTCW,Z0,
     &   FALL,RHEIGHT,RLENGTH,C0A,QS,QSDIR,SEDM,SED,SEDDIR,
     &   AB,WL,USTCRB,USTCRS,USTUP,Z0C,PHIB,PHI100,DELTACW,USTCS,
     &   USTWS,USTCWSE,USTCWSB,USTC,USTW,RPLCOEF,TB1,TB2,TS1,TS2,
     &   PERBED,PERSUSP,C0,TAOCWS,RHOW,VISC,
     &   " ",CH2(1:LASTCH(CH2))
         CALL LINETAB(CH)
         WRITE(12,'(A)',ERR=320)CH(1:LASTCH(CH))
      ELSE
C write the tabulated output to the *.dat file
         WRITE(CH,*)IRUN,ERR(1),UB,FCW,UA,U100,USTCWS,USTCW,Z0,
     &   FALL,RHEIGHT,RLENGTH,C0A,QS,QSDIR,SEDM,SED,SEDDIR
         CALL LINETAB(CH)
         WRITE(12,'(A)',ERR=320)CH(1:LASTCH(CH))

C write the additional tabulated output to the *.da2 file
         WRITE(CH,*)IRUN,
     &   AB,WL,USTCRB,USTCRS,USTUP,Z0C,PHIB,PHI100,DELTACW,USTCS,
     &   USTWS,USTCWSE,USTCWSB,USTC,USTW,RPLCOEF,TB1,TB2,TS1,TS2,
     &   PERBED,PERSUSP,C0,TAOCWS,RHOW,VISC
         CALL LINETAB(CH)
         WRITE(13,'(A)',ERR=330)CH(1:LASTCH(CH))

C write the profile of SSC and U for non-cohesive sediment to the *.pro file
         IF(IOPT1.NE.7) THEN
            DO 50, I=1,NBPROF
               Z2(I)=ZZ(I);
50          CONTINUE
            WRITE(CH,*)IRUN,NBPROF,Z2,CZ,UUZ
         ELSE                     ! No profile for cohesive sediment
            WRITE(CH,*)IRUN,NBPROF,(0,I=1,NBPROF*3)
         ENDIF
         CALL LINETAB(CH)
         WRITE(14,'(A)',ERR=340)CH(1:LASTCH(CH))
      ENDIF

C write the cohesive output to the *.coh file
      IF (IOPT1.EQ.7) THEN
C set unused rows of BEDCHA to zero
         DO 200, I=2,MAXBED
            IF(BEDCHA(I,1).EQ.0) GOTO 210
200      CONTINUE
210      CONTINUE
         DO 220, J=I,MAXBED
            BEDCHA(J,2)=0d0
            BEDCHA(J,3)=0d0
220      CONTINUE

         WRITE(CH,*)IRUN,ERR(1),TAU0EFF,TAOS,0,ZS,EDRATE,TCONC,NBCONC,
     &      (CONC(I),I=1,NBCONC),MAXBED,(BEDCHA(I,1),I=1,MAXBED),
     &      (BEDCHA(I,2),I=1,MAXBED),(BEDCHA(I,3),I=1,MAXBED)
         CALL LINETAB(CH)
         WRITE(15,'(A)',ERR=350)CH(1:LASTCH(CH))
      ENDIF

      RETURN

300   WRITE(*,*) 'Error during opening file ',OUT5
      STOP
310   WRITE(*,*) 'Error during writing to file ',OUTFILE, '.txt'
      STOP
320   WRITE(*,*) 'Error during writing to file ',OUTFILE, '.dat'
      STOP
330   WRITE(*,*) 'Error during writing to file ',OUTFILE, '.da2'
      STOP
340   WRITE(*,*) 'Error during writing to file ',OUTFILE, '.pro'
      STOP
350   WRITE(*,*) 'Error during writing to file ',OUT5
      STOP

      END   ! end of subroutine WRITEOUT



      INCLUDE 'cohesive.f'
      INCLUDE 'bedform.f'
      INCLUDE 'sedtrans-core.f'
